@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -22,17 +22,8 @@ IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-
-#include <netdude/nd.h>
-#include <netdude/nd_debug.h>
-#include <netdude/nd_gui.h>
-#include <netdude/nd_protocol_registry.h>
-#include <netdude/nd_misc.h>
-#include <netdude/nd_raw_protocol.h>
-#include <netdude/nd_gettext.h>
+#include <nd.h>
+#include <nd_gui.h>
 
 #include <nd_icmp.h>
 #include <nd_icmp_callbacks.h>
@@ -148,75 +139,8 @@ ND_MenuData icmp_menu_data[] = {
 };
 
 
-static ND_Protocol *icmp;
-
-
-static ND_Protocol *
-icmp_get_ip(void)
-{
-  static ND_Protocol *ip = NULL;
-
-  if (!ip)
-    {
-      ip = nd_proto_registry_find(ND_PROTO_LAYER_NET, 0x0800);
-      
-      if (ip->id == 1)
-	ip = NULL;
-    }
-
-  return ip;
-}
-
-
-static struct ip *
-icmp_get_last_ip_before_icmp(const ND_Packet *packet, struct icmphdr **icmphdr)
-{
-  ND_Protocol *ip;
-  GList *l;
-  ND_ProtoData *pd;
-  struct ip *iphdr = NULL;
-
-  if (!packet || !(ip = icmp_get_ip()))
-    return NULL;
-
-  for (l = packet->pd; l; l = g_list_next(l))
-    {
-      pd = (ND_ProtoData *) l->data;
-
-      if (pd->inst.proto == icmp)
-	{
-	  if (icmphdr)
-	    *icmphdr = (struct icmphdr*) pd->data;
-	  break;
-	}
-
-      if (pd->inst.proto == ip)
-	iphdr = (struct ip*) pd->data;
-    }
-
-  return iphdr;
-}
-
-
-/**
- * icmp_header_complete -
- *
- * checks whether the ICMP message is complete by looking at the
- * IP length field -- easier since ICMP messages can be of varying sizes.
- */
-static gboolean
-icmp_header_complete(const ND_Packet *packet)
-{
-  struct ip *iphdr = NULL;
-
-  iphdr = icmp_get_last_ip_before_icmp(packet, NULL);
-  if (!iphdr)
-    return FALSE;
-
-  return (((guchar*) iphdr) + ntohs(iphdr->ip_len) <= nd_packet_get_end(packet));
-}
-
-
+static LND_Protocol *icmp;
+static ND_Protocol  *icmp_gui;
 
 
 /* Plugin hook implementations: ---------------------------------------- */
@@ -245,29 +169,29 @@ author(void)
 const char *
 version(void)
 {
-  return ("0.1.0");
+  return VERSION_MAJOR;
 }
 
 
-ND_Protocol *
+LND_Protocol *
 init(void)
 {
-  icmp = nd_proto_new("ICMP", ND_PROTO_LAYER_NET | ND_PROTO_LAYER_TRANS, IPPROTO_ICMP);
-			     		
-  icmp->create_gui      = nd_icmp_create_gui;
-  icmp->set_gui         = nd_icmp_set_gui;
-  icmp->init_packet     = nd_icmp_init_packet;
-  icmp->header_complete = nd_icmp_header_complete;
-  icmp->fix_packet      = nd_icmp_fix_packet;
+  if (! (icmp = libnd_proto_registry_find(LND_PROTO_LAYER_NET|LND_PROTO_LAYER_TRANS,
+					  IPPROTO_ICMP)))
+    return NULL;
+  
+  icmp_gui = nd_proto_new(icmp);
+
+  icmp_gui->create_gui      = nd_icmp_create_gui;
+  icmp_gui->set_gui         = nd_icmp_set_gui;
 
   /* We're using a button table to display the protocol content,
      so we need to hook them in here: */
-  icmp->fields = icmp_fields;
-
-  icmp->header_width = 32;
+  icmp_gui->fields = icmp_fields;
+  icmp_gui->header_width = 32;
 
   /* We provide a little menu that allows us to fix checksums. */
-  icmp->menu = nd_gui_create_menu(icmp_menu_data);
+  icmp_gui->menu = nd_gui_create_menu(icmp_menu_data);
 
   return icmp;
 }
@@ -276,7 +200,7 @@ init(void)
 /* Protocol method implementations: ------------------------------------ */
 
 GtkWidget *
-nd_icmp_create_gui(ND_Trace *trace, ND_ProtoInfo *pinf)
+nd_icmp_create_gui(LND_Trace *trace, LND_ProtoInfo *pinf)
 {
   GtkWidget *table;
 
@@ -287,23 +211,23 @@ nd_icmp_create_gui(ND_Trace *trace, ND_ProtoInfo *pinf)
 
 
 void       
-nd_icmp_set_gui(const ND_Packet *packet, ND_ProtoInfo *pinf)
+nd_icmp_set_gui(const LND_Packet *packet, LND_ProtoInfo *pinf)
 {
-  struct icmphdr *icmphdr;
-
-  icmphdr = (struct icmphdr*) nd_packet_get_data(packet, icmp, 0);
+  struct icmp *icmphdr;
+  
+  icmphdr = (struct icmp*) libnd_packet_get_data(packet, icmp, 0);
 
   nd_icmp_set_gui_type(pinf, icmphdr);
   nd_icmp_set_gui_code(pinf, icmphdr);
   nd_icmp_set_gui_cksum(pinf, icmphdr, packet);
 
-  if (nd_icmp_header_is_error(icmphdr))
+  if (libnd_icmp_header_is_error(icmphdr))
     {
       nd_icmp_set_gui_data(pinf, icmphdr, packet);
       return;
     }
 
-  switch (icmphdr->type)
+  switch (icmphdr->icmp_type)
     {
     case ICMP_ECHO:
     case ICMP_ECHOREPLY:
@@ -334,250 +258,60 @@ nd_icmp_set_gui(const ND_Packet *packet, ND_ProtoInfo *pinf)
       break;
 
     default:
+      D(("Unhandled ICMP code\n"));
     }
 }
 
 
-void       
-nd_icmp_init_packet(ND_Packet *packet, guchar *data, guchar *data_end)
-{
-  ND_Protocol *ip;
-  struct icmphdr *icmphdr;
-  struct ip *iphdr;
-
-  /* Cast the data pointer into your protocol's header */
-  icmphdr = (struct icmphdr *) data;
-
-  if (!icmp_header_complete(packet))
-    {
-      nd_raw_proto_get()->init_packet(packet, data, data_end);
-      return;
-    }
-
-  nd_packet_add_proto_data(packet, icmp, data, data_end);  
-
-  /* We'll need IP data below, make sure we can find the IP protocol.
-   */
-  if (! (ip = icmp_get_ip()))
-    {
-      D(("WARNING -- IP not found\n"));
-      return;
-    }
-
-  /* For some ICMP messages we offer the hex editor to edit the data.
-   * Otherwise, we do nothing, as ICMP is the innermost protocol. */
-
-  if (nd_icmp_header_is_error(icmphdr))
-    {      
-      ip->init_packet(packet, data + 8, data_end);
-      return;
-    }
-  
-  iphdr = icmp_get_last_ip_before_icmp(packet, NULL);
-  if (!iphdr)
-    return;
-
-  if (((guchar*) icmphdr) + 8 >= ((guchar*) iphdr) + ntohs(iphdr->ip_len))
-    return;
-
-  D(("Pointers: %p %p, difference: %u\n", 
-     (((guchar*) icmphdr) + 8), (((guchar*) iphdr) + ntohs(iphdr->ip_len)),
-     (((guchar*) iphdr) + ntohs(iphdr->ip_len)) - (((guchar*) icmphdr) + 8)));
-
-  switch (icmphdr->type)
-    {
-      /* Echo requests and replies can have optional data that must
-	 be echoed. we display that using the raw data plugin:
-      */
-    case ICMP_ECHO:
-    case ICMP_ECHOREPLY:
-      nd_raw_proto_get()->init_packet(packet, data + 8, data_end);
-    }
-}
-
-
-gboolean
-nd_icmp_header_complete(const ND_Packet *packet, guint nesting)
-{
-  if (!packet)
-    return FALSE;
-
-  return icmp_header_complete(packet);
-  TOUCH(nesting);
-}
-
-
-gboolean
-nd_icmp_message_complete(const ND_Packet *packet)
-{
-  struct icmphdr *icmphdr;
-  struct ip *iphdr;
-
-  if (!packet)
-    return FALSE;
-
-  icmphdr = (struct icmphdr *) nd_packet_get_data(packet, icmp, 0);
-
-  if (!icmphdr)
-    return FALSE;
-
-  /* If it's an error message, check whether we have the full
-   * datagram that contains the ICMP message.
-   */
-  if (nd_icmp_header_is_error(icmphdr))
-    {
-      iphdr = (struct ip *) ((guchar *) icmphdr + 8);
-      
-      return ((guchar *) iphdr + (iphdr->ip_hl << 2) + 8 <= nd_packet_get_end(packet));
-    }
-
-  /* For all other cases, check individually.
-   */
-  switch (icmphdr->type)
-    {
-    case ICMP_ECHO:
-    case ICMP_ECHOREPLY:
-
-      iphdr = icmp_get_last_ip_before_icmp(packet, NULL);
-      if (!iphdr)
-	return FALSE;
-
-      return ((guchar *) iphdr + ntohs(iphdr->ip_len) <= nd_packet_get_end(packet));
-
-    case ICMP_TIMESTAMP:
-    case ICMP_TIMESTAMPREPLY:
-      return ((guchar *) icmphdr + 20 <= nd_packet_get_end(packet));
-
-    case ICMP_INFO_REQUEST:
-    case ICMP_INFO_REPLY:
-    case 10:
-      return ((guchar *) icmphdr + 8 <= nd_packet_get_end(packet));
-
-    case ICMP_ADDRESS:
-    case ICMP_ADDRESSREPLY:
-      return ((guchar *) icmphdr + 12 <= nd_packet_get_end(packet));
-
-    case 9:
-      {
-	struct nd_icmphdr *icmphdr_adv = (struct nd_icmphdr *) icmphdr;
-
-	return ((guchar *) icmphdr + 8 + icmphdr_adv->un.adv.num_addr * 8 <= nd_packet_get_end(packet));
-      }
-
-    default:
-    }
-
-  return FALSE;
-}
-
-
-guint16    
-nd_icmp_checksum(const ND_Packet *packet)
-{
-  guint16         old_sum, sum;
-  struct icmphdr *icmphdr;
-  struct ip      *iphdr;
-
-  
-  iphdr = icmp_get_last_ip_before_icmp(packet, &icmphdr);
-  
-  old_sum = icmphdr->checksum;
-  icmphdr->checksum = 0;
-  sum = nd_misc_in_cksum((u_short *)icmphdr, ntohs(iphdr->ip_len) - (iphdr->ip_hl << 2), 0);
-  icmphdr->checksum = old_sum;
-  
-  return sum;
-}
-
-
-gboolean   
-nd_icmp_csum_correct(const ND_Packet *packet, guint16 *correct_sum)
-{
-  struct icmphdr *icmphdr;
-  guint16 sum;
-
-  if (!packet)
-    return FALSE;
-
-  icmphdr = (struct icmphdr *) nd_packet_get_data(packet, icmp, 0);
-  
-  sum = nd_icmp_checksum(packet);
-  
-  if (correct_sum)
-    *correct_sum = sum;
-  
-  return (icmphdr->checksum == sum);
-}
-
-
-gboolean
-nd_icmp_fix_packet(ND_Packet *packet, int index)
-{
-  struct icmphdr *icmphdr;
-  guint16         correct_sum;
-  
-  if (!packet)
-    return FALSE;
-  
-  if (!nd_icmp_csum_correct(packet, &correct_sum))
-    {
-      icmphdr = (struct icmphdr*) nd_packet_get_data(packet, icmp, 0);
-      icmphdr->checksum = correct_sum;
-      nd_packet_modified_at_index(packet, index);
-      return TRUE;
-    }
-  
-  return FALSE;
-}
 
 /* Misc helper stuff below --------------------------------------------- */
 
 void
-nd_icmp_set_gui_type(ND_ProtoInfo *pinf, struct icmphdr *icmphdr)
+nd_icmp_set_gui_type(LND_ProtoInfo *pinf, struct icmp *icmphdr)
 {
-  nd_proto_field_set_for_menu(pinf, &icmp_fields[0], DATA_TO_PTR(icmphdr->type),
+  nd_proto_field_set_for_menu(pinf, &icmp_fields[0], DATA_TO_PTR(icmphdr->icmp_type),
 			      icmp_menu_type_data, "%u");
 }
 
 
 void
-nd_icmp_set_gui_code(ND_ProtoInfo *pinf, struct icmphdr *icmphdr)
+nd_icmp_set_gui_code(LND_ProtoInfo *pinf, struct icmp *icmphdr)
 {
   char            s[MAXPATHLEN];
   
-  switch (icmphdr->type)
+  switch (icmphdr->icmp_type)
     {
     case ICMP_DEST_UNREACH:
-      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->code),
+      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->icmp_code),
 				  icmp_menu_unreach_code_data, "%u");
       break;
     case ICMP_REDIRECT:
-      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->code),
+      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->icmp_code),
 				  icmp_menu_redirect_code_data, "%u");
       break;
     case ICMP_TIME_EXCEEDED:
-      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->code),
+      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->icmp_code),
 				  icmp_menu_time_ex_code_data, "%u");
       break;
     case ICMP_PARAMETERPROB:
-      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->code),
+      nd_proto_field_set_for_menu(pinf, &icmp_fields[1], DATA_TO_PTR(icmphdr->icmp_code),
 				  icmp_menu_param_code_data, "%u");
       break;
     default:
-      g_snprintf(s, MAXPATHLEN, "%u", icmphdr->code);
+      g_snprintf(s, MAXPATHLEN, "%u", icmphdr->icmp_code);
       nd_proto_field_set(pinf, &icmp_fields[1], s);
     }
 }
 
 
 void
-nd_icmp_set_gui_cksum(ND_ProtoInfo *pinf,
-		      struct icmphdr *icmphdr,
-		      const ND_Packet *packet)
+nd_icmp_set_gui_cksum(LND_ProtoInfo *pinf,
+		      struct icmp *icmphdr,
+		      const LND_Packet *packet)
 {
-  nd_proto_field_set(pinf, &icmp_fields[2], DATA_TO_PTR(ntohs(icmphdr->checksum)));
+  nd_proto_field_set(pinf, &icmp_fields[2], DATA_TO_PTR(ntohs(icmphdr->icmp_cksum)));
 
-  if (!nd_icmp_message_complete(packet))
+  if (!libnd_icmp_message_complete(packet))
     {
       nd_proto_info_field_set_state(pinf, 
 				    &icmp_fields[2],
@@ -585,7 +319,7 @@ nd_icmp_set_gui_cksum(ND_ProtoInfo *pinf,
       return;
     }
   
-  if (!nd_icmp_csum_correct(packet, NULL))
+  if (!libnd_icmp_csum_correct(packet, NULL))
     nd_proto_info_field_set_state(pinf, 
 				  &icmp_fields[2],
 				  ND_FIELD_STATE_ERROR);
@@ -597,135 +331,147 @@ nd_icmp_set_gui_cksum(ND_ProtoInfo *pinf,
 
 
 void
-nd_icmp_set_gui_data(ND_ProtoInfo *pinf,
-		     struct icmphdr *icmphdr,
-		     const ND_Packet *packet)
+nd_icmp_set_gui_data(LND_ProtoInfo *pinf,
+		     struct icmp *icmphdr,
+		     const LND_Packet *packet)
 {
   char            s[MAXPATHLEN];
   guchar         *data;
 
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
   
   data = ((guchar*) icmphdr) + 4;
   g_snprintf(s, MAXPATHLEN, "%.2x:%.2x:%.2x:%.2x", data[0], data[1], data[2], data[3]);	     
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_error_fields[0], s, FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_error_fields[0], s, FALSE);
 }
 
 
 void
-nd_icmp_set_gui_echo(ND_ProtoInfo *pinf,
-		     struct icmphdr *icmphdr,
-		     const ND_Packet *packet)
+nd_icmp_set_gui_echo(LND_ProtoInfo *pinf,
+		     struct icmp *icmphdr,
+		     const LND_Packet *packet)
 {
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[0], DATA_TO_PTR(ntohs(icmphdr->un.echo.id)), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[1], DATA_TO_PTR(ntohs(icmphdr->un.echo.sequence)), FALSE);  
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[0],
+			 DATA_TO_PTR(icmphdr->icmp_id), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[1],
+			 DATA_TO_PTR(icmphdr->icmp_seq), FALSE);  
 }
 
 
 void
-nd_icmp_set_gui_timestamp(ND_ProtoInfo *pinf,
-			  struct icmphdr *icmphdr,
-			  const ND_Packet *packet)
+nd_icmp_set_gui_timestamp(LND_ProtoInfo *pinf,
+			  struct icmp *icmphdr,
+			  const LND_Packet *packet)
 {
-  struct nd_icmphdr *icmphdr_ts = (struct nd_icmphdr *) icmphdr;
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
 
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[0],
+			 DATA_TO_PTR(icmphdr->icmp_id), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[1],
+			 DATA_TO_PTR(icmphdr->icmp_seq), FALSE);  
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[0], DATA_TO_PTR(ntohs(icmphdr->un.echo.id)), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[1], DATA_TO_PTR(ntohs(icmphdr->un.echo.sequence)), FALSE);  
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_tstamp_fields[0],
+			 DATA_TO_PTR(ntohl(icmphdr->icmp_otime)), FALSE);
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_tstamp_fields[0],
-			 DATA_TO_PTR(ntohl(icmphdr_ts->un.ts.orig_ts)), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_tstamp_fields[1],
+			 DATA_TO_PTR(ntohl(icmphdr->icmp_rtime)), FALSE);  
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_tstamp_fields[1],
-			 DATA_TO_PTR(ntohl(icmphdr_ts->un.ts.recv_ts)), FALSE);  
-
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_tstamp_fields[2],
-			 DATA_TO_PTR(ntohl(icmphdr_ts->un.ts.trans_ts)), FALSE);    
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_tstamp_fields[2],
+			 DATA_TO_PTR(ntohl(icmphdr->icmp_ttime)), FALSE);    
 }
 
 
 void
-nd_icmp_set_gui_info(ND_ProtoInfo *pinf,
-		     struct icmphdr *icmphdr,
-		     const ND_Packet *packet)
+nd_icmp_set_gui_info(LND_ProtoInfo *pinf,
+		     struct icmp *icmphdr,
+		     const LND_Packet *packet)
 {
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[0], DATA_TO_PTR(ntohs(icmphdr->un.echo.id)), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[1], DATA_TO_PTR(ntohs(icmphdr->un.echo.sequence)), FALSE);  
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[0],
+			 DATA_TO_PTR(icmphdr->icmp_id), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[1],
+			 DATA_TO_PTR(icmphdr->icmp_seq), FALSE);  
 }
 
 
 void
-nd_icmp_set_gui_addr(ND_ProtoInfo *pinf,
-		     struct icmphdr *icmphdr,
-		     const ND_Packet *packet)
+nd_icmp_set_gui_addr(LND_ProtoInfo *pinf,
+		     struct icmp *icmphdr,
+		     const LND_Packet *packet)
 {
-  struct nd_icmphdr *icmphdr_in = (struct nd_icmphdr *) icmphdr;
-  
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[0], DATA_TO_PTR(ntohs(icmphdr->un.echo.id)), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_req_seq_fields[1], DATA_TO_PTR(ntohs(icmphdr->un.echo.sequence)), FALSE);  
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[0],
+			 DATA_TO_PTR(icmphdr->icmp_id), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_req_seq_fields[1],
+			 DATA_TO_PTR(icmphdr->icmp_seq), FALSE);  
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_addr_mask_fields[0],
-			 inet_ntoa(icmphdr_in->un.in.in), FALSE);    
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_addr_mask_fields[0],
+			 inet_ntoa(icmphdr->icmp_gwaddr), FALSE);    
 }
 
 
 void       
-nd_icmp_set_gui_router_adv(ND_ProtoInfo *pinf,
-			   struct icmphdr *icmphdr,
-			   const ND_Packet *packet)
+nd_icmp_set_gui_router_adv(LND_ProtoInfo *pinf,
+			   struct icmp *icmphdr,
+			   const LND_Packet *packet)
 {
   int i;
-  struct nd_icmphdr *icmphdr_adv = (struct nd_icmphdr *) icmphdr;
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
 
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_router_adv_fields[0],
+			 DATA_TO_PTR(icmphdr->icmp_num_addrs), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_router_adv_fields[1],
+			 DATA_TO_PTR(icmphdr->icmp_wpa), FALSE);
+  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_router_adv_fields[2],
+			 DATA_TO_PTR(ntohs(icmphdr->icmp_lifetime)), FALSE);
 
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_router_adv_fields[0], DATA_TO_PTR(icmphdr_adv->un.adv.num_addr), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_router_adv_fields[1], DATA_TO_PTR(icmphdr_adv->un.adv.addr_size), FALSE);
-  nd_gui_proto_table_add(packet->trace, pinf, &icmp_router_adv_fields[2], DATA_TO_PTR(ntohs(icmphdr_adv->un.adv.lifetime)), FALSE);
-
-  for (i = 0; i < icmphdr_adv->un.adv.num_addr; i++)
+  for (i = 0; i < icmphdr->icmp_num_addrs; i++)
     {
       guchar *data;
       struct in_addr *in;
       guint32 *prec;
 
-      data = &icmphdr_adv->un.adv.addr_data + i * 8;
+      data = ((guchar *) (&icmphdr->icmp_lifetime + 1)) + i * 8;
 
-      if (data + 8 > nd_packet_get_end(packet))
+      if (data + 8 > libnd_packet_get_end(packet))
 	break;
 
       in = (struct in_addr *) data;
       prec = (guint32 *) (data + 4);
 
-      nd_gui_proto_table_add(packet->trace, pinf, &icmp_router_adv_fields[3],
+      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_router_adv_fields[3],
 			     inet_ntoa(*in), FALSE);
-      nd_gui_proto_table_add(packet->trace, pinf, &icmp_router_adv_fields[4],
+      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &icmp_router_adv_fields[4],
 			     DATA_TO_PTR(ntohl(*prec)), FALSE);
     }
 }
 
 
-ND_Protocol *
+LND_Protocol *
 nd_icmp_get(void)
 {
   return icmp;
 }
 
 
+ND_Protocol *
+nd_icmp_get_gui(void)
+{
+  return icmp_gui;
+}
+
+
 gboolean
-nd_icmp_header_is_error(struct icmphdr *icmphdr)
+nd_icmp_header_is_error(struct icmp *icmphdr)
 {
   if (!icmphdr)
     return FALSE;
 
-  switch (icmphdr->type)
+  switch (icmphdr->icmp_type)
     {
     case ICMP_DEST_UNREACH:
     case ICMP_SOURCE_QUENCH:
@@ -737,3 +483,15 @@ nd_icmp_header_is_error(struct icmphdr *icmphdr)
 
   return FALSE;
 }
+
+
+
+
+
+
+
+
+
+
+
+
