@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -23,15 +23,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-#include <netdude/nd_debug.h>
-#include <netdude/nd_protocol_registry.h>
-#include <netdude/nd_misc.h>
-#include <netdude/nd_raw_protocol.h>
-#include <netdude/nd_protocol_inst.h>
-#include <netdude/nd.h>
+#include <nd.h>
+#include <nd_gui.h>
+#include <nd_protocol.h>
+#include <nd_raw_protocol.h>
+#include <nd_protocol_inst.h>
 
-#include <net/ethernet.h>
-#include <arpa/inet.h>
+#include <netdb.h>
+#include <net/if.h>
+#include <netinet/if_ether.h>
 #include <time.h>
 
 #include <nd_ip.h>
@@ -126,14 +126,8 @@ ND_MenuData ip_menu_data[] = {
 };
 
 
-static ND_Protocol *ip;
-
-
-static gboolean
-ip_header_complete(const ND_Packet *packet, struct ip *iphdr)
-{
-  return (((guchar*) iphdr) + 4 * iphdr->ip_hl <= nd_packet_get_end(packet));
-}
+static LND_Protocol *ip;
+static ND_Protocol  *ip_gui;
 
 
 /* Plugin hook implementations: ---------------------------------------- */
@@ -165,36 +159,32 @@ author(void)
 const char *
 version(void)
 {
-  return ("0.1.0");
+  return VERSION_MAJOR;
 }
 
 
-ND_Protocol *
+LND_Protocol *
 init(void)
 {
-  ip = nd_proto_new("IPv4", ND_PROTO_LAYER_NET, ETHERTYPE_IP);
-	
-  ip->create_gui      = nd_ip_create_gui;
-  ip->set_gui         = nd_ip_set_gui;
-  ip->init_packet     = nd_ip_init_packet;
-  ip->header_complete = nd_ip_header_complete;
-  ip->fix_packet      = nd_ip_fix_packet;
+  if (! (ip = libnd_proto_registry_find(LND_PROTO_LAYER_NET, ETHERTYPE_IP)))
+    return NULL;
 
-  /* This one isn't really necessary, just for illustration */
-  ip->is_stateful = FALSE;
+  ip_gui = nd_proto_new(ip);
+	
+  ip_gui->create_gui      = nd_ip_create_gui;
+  ip_gui->set_gui         = nd_ip_set_gui;
 
   /* We're using a button table to display the protocol content,
      so we need to hook it in here: */
-  ip->fields = ip_fields;
-
-  ip->header_width = 32;
+  ip_gui->fields = ip_fields;
+  ip_gui->header_width = 32;
 
   /* IP isn't stateful, so we don't need any of the state-related
      functions. */
 
   /* We provide a little menu that allows us to fragment/defragment
      packets, and fix checksums. */
-  ip->menu = nd_gui_create_menu(ip_menu_data);
+  ip_gui->menu = nd_gui_create_menu(ip_menu_data);
 	     
   return ip;
 }
@@ -203,7 +193,7 @@ init(void)
 /* Protocol method implementations: ------------------------------------ */
 
 GtkWidget *
-nd_ip_create_gui(ND_Trace *trace, ND_ProtoInfo *pinf)
+nd_ip_create_gui(LND_Trace *trace, LND_ProtoInfo *pinf)
 {
   GtkWidget *table;
 
@@ -213,11 +203,13 @@ nd_ip_create_gui(ND_Trace *trace, ND_ProtoInfo *pinf)
 
 
 void       
-nd_ip_set_gui(const ND_Packet *packet, ND_ProtoInfo *pinf)
+nd_ip_set_gui(const LND_Packet *packet, LND_ProtoInfo *pinf)
 {
   struct ip   *iphdr;
+  LND_ProtoData *pd;
+  struct protoent *pe;
 
-  iphdr = (struct ip*) nd_packet_get_data(packet, ip, pinf->inst.nesting);
+  iphdr = (struct ip*) libnd_packet_get_data(packet, ip, pinf->inst.nesting);
   if (!iphdr)
     return;
 
@@ -239,149 +231,48 @@ nd_ip_set_gui(const ND_Packet *packet, ND_ProtoInfo *pinf)
   nd_ip_set_gui_src(pinf, iphdr);
   nd_ip_set_gui_dst(pinf, iphdr);
   nd_ip_set_gui_options(pinf, iphdr, packet);
-}
-
-
-void       
-nd_ip_init_packet(ND_Packet *packet, u_char *data, u_char *data_end)
-{
-  ND_Protocol *payload_proto = NULL;
-  struct ip* iphdr = (struct ip*) data;
-
-
-  if (!ip_header_complete(packet, iphdr))
-    {
-      nd_raw_proto_get()->init_packet(packet, data, data_end);
-      return;
-    }
-
-  nd_packet_add_proto_data(packet, ip, data, data + ntohs(iphdr->ip_len));
-
-  /* If this is a first fragment or normal packet, check contents
-     and delegate initialization. */
-
-  if ((ntohs(iphdr->ip_off) & IP_OFFMASK) == 0 &&
-      (ntohs(iphdr->ip_off) & IP_MF) == 0)
-    {
-      payload_proto = nd_proto_registry_find(ND_PROTO_LAYER_NET | ND_PROTO_LAYER_TRANS,
-					     iphdr->ip_p);
-
-      /* Adjust the data pointers so that we only pass the really
-	 present data end. But IP may be nested, so we cannot rely
-	 on the ip_len field to be relevant for this packet, so
-	 sanity-check against end of captured data!
-      */
-
-      payload_proto->init_packet(packet,
-				 data + 4 * iphdr->ip_hl,
-				 MIN(data_end, data + ntohs(iphdr->ip_len)));
-    }
-  else
-    {
-      /* It's fragment data -- display in raw data displayer */
-
-      nd_raw_proto_get()->init_packet(packet,
-				      data + 4 * iphdr->ip_hl,
-				      MIN(data_end, data + ntohs(iphdr->ip_len)));
-    }
-
-  return;
-  TOUCH(data_end);
-}
-
-
-gboolean
-nd_ip_header_complete(const ND_Packet *packet, guint nesting)
-{
-  struct ip *iphdr;
-
-  if (!packet)
-    return FALSE;
-
-  iphdr = (struct ip*) nd_packet_get_data(packet, ip, nesting);
-
-  return ip_header_complete(packet, iphdr);
-}
-
-
-guint16
-nd_ip_checksum(const struct ip* iphdr)
-{
-  guint16    old_sum, sum;
-  struct ip *iphdr_work = (struct ip*) iphdr;
-
-  old_sum = iphdr->ip_sum;
-  iphdr_work->ip_sum = 0;
-  sum = nd_misc_in_cksum((u_short *)iphdr_work, iphdr->ip_hl << 2, 0);
-  iphdr_work->ip_sum = old_sum;
-
-  return sum;
-}
-
-
-static void
-ip_fix_cb(ND_Packet    *packet,
-	  ND_ProtoData *pd,
-	  void         *user_data)
-{
-  gboolean  *modified = (gboolean *) user_data;
-  guint16    correct_sum;
-  struct ip *iphdr;
 
+  /* Try to label the next tab appropriately if it's not a handled
+   * protocol (e.g., "HTTP" is nicer than saying "(remaining)").
+   *
+   * First check if there actually is uninterpreted data and
+   * TCP coming before it, and if we can then find a string
+   * representation of the port number, replace the label of
+   * the raw data displayer with that service name.
+   */
+  if (! (pd = libnd_packet_get_last_nonraw(packet)) || !libnd_packet_get_trace(packet))
+    return;
+  
   if (pd->inst.proto != ip)
     return;
 
-  iphdr = (struct ip*) pd->data;
-            
-  /* Recalculate IP checksum, and if it changed,
-     tell Netdude that we modified the packet. */
-  
-  if (!nd_ip_csum_correct(iphdr, &correct_sum))
+  if ( (pe = getprotobynumber(iphdr->ip_p)))
     {
-      iphdr->ip_sum = correct_sum;
-      *modified = TRUE;
+      ND_ProtoInfo *pinf_raw = nd_raw_proto_get_gui(libnd_packet_get_trace(packet));
+      gtk_label_set_text(GTK_LABEL(pinf_raw->proto_label), pe->p_name);
     }
-
-  return;
-  TOUCH(packet);
 }
 
 
-gboolean
-nd_ip_fix_packet(ND_Packet *packet, int index)
-{
-  gboolean modified = FALSE;
-
-  if (!packet)
-    return FALSE;
-
-  nd_packet_foreach_proto_backward(packet, ip_fix_cb, &modified);
-
-  if (modified)
-    nd_packet_modified_at_index(packet, index);
-
-  return modified;
-}
-
   
 /* Misc helper stuff below --------------------------------------------- */
 
 void       
-nd_ip_set_gui_v(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_v(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[0], DATA_TO_PTR(iphdr->ip_v));
 }
 
 
 void       
-nd_ip_set_gui_hl(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_hl(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[1], DATA_TO_PTR(iphdr->ip_hl));
 }
 
 
 void       
-nd_ip_set_gui_ecn(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_ecn(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set_for_menu(pinf, &ip_fields[2], DATA_TO_PTR(iphdr->ip_tos >> 6),
 			      ip_menu_ecn_data, "%u");
@@ -390,7 +281,7 @@ nd_ip_set_gui_ecn(ND_ProtoInfo *pinf, struct ip *iphdr)
 
 
 void       
-nd_ip_set_gui_tos(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_tos(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set_for_menu(pinf, &ip_fields[3], DATA_TO_PTR(iphdr->ip_tos),
 			      ip_menu_tos_data, "0x%.2x");
@@ -399,56 +290,56 @@ nd_ip_set_gui_tos(ND_ProtoInfo *pinf, struct ip *iphdr)
 
 
 void       
-nd_ip_set_gui_len(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_len(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[4], DATA_TO_PTR(ntohs(iphdr->ip_len)));
 }
 
 
 void       
-nd_ip_set_gui_id(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_id(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[5], DATA_TO_PTR(ntohs(iphdr->ip_id)));
 }
 
 
 void       
-nd_ip_set_gui_rf(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_rf(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[6], DATA_TO_PTR(ntohs(iphdr->ip_off) & IP_RF));
 }
 
 
 void       
-nd_ip_set_gui_df(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_df(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[7], DATA_TO_PTR(ntohs(iphdr->ip_off) & IP_DF));
 }
 
 
 void       
-nd_ip_set_gui_mf(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_mf(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[8], DATA_TO_PTR(ntohs(iphdr->ip_off) & IP_MF));
 }
 
 
 void       
-nd_ip_set_gui_off(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_off(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[9], DATA_TO_PTR(ND_IP_FRAG_OFFSET(iphdr)));
 }
 
 
 void       
-nd_ip_set_gui_ttl(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_ttl(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[10], DATA_TO_PTR(iphdr->ip_ttl));
 }
 
 
 void       
-nd_ip_set_gui_p(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_p(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set_for_menu(pinf, &ip_fields[11], DATA_TO_PTR(iphdr->ip_p),
 			      ip_menu_p_data, "%i");
@@ -456,11 +347,11 @@ nd_ip_set_gui_p(ND_ProtoInfo *pinf, struct ip *iphdr)
 
 
 void       
-nd_ip_set_gui_sum(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_sum(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[12], DATA_TO_PTR(ntohs(iphdr->ip_sum)));
   
-  if (!nd_ip_csum_correct(iphdr, NULL))
+  if (!libnd_ip_csum_correct(iphdr, NULL))
     nd_proto_info_field_set_state(pinf, 
 				  &ip_fields[12],
 				  ND_FIELD_STATE_ERROR);
@@ -472,22 +363,22 @@ nd_ip_set_gui_sum(ND_ProtoInfo *pinf, struct ip *iphdr)
 
 
 void       
-nd_ip_set_gui_src(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_src(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[13], inet_ntoa(iphdr->ip_src));
 }
 
 
 void       
-nd_ip_set_gui_dst(ND_ProtoInfo *pinf, struct ip *iphdr)
+nd_ip_set_gui_dst(LND_ProtoInfo *pinf, struct ip *iphdr)
 {
   nd_proto_field_set(pinf, &ip_fields[14], inet_ntoa(iphdr->ip_dst));
 }
 
 
 void
-nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
-		      const ND_Packet *packet)
+nd_ip_set_gui_options(LND_ProtoInfo *pinf, struct ip *iphdr,
+		      const LND_Packet *packet)
 {
   int          opts_len, o_len, val, opts_done, opts_done_old;
   guchar      *opts_p, *o_next;
@@ -497,7 +388,7 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
   if (!iphdr)
     return;
   
-  nd_gui_proto_table_clear(packet->trace, pinf);
+  nd_gui_proto_table_clear(libnd_packet_get_trace(packet), pinf);
   
   /* Setup options GUI elements manually: */
   
@@ -519,7 +410,7 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	case 0:
 	  /* end of option list - RFC791 */
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("EOL"), FALSE);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("EOL"), FALSE);
 	  opts_p++;
 	  opts_done = opts_len;
 	  break;
@@ -527,7 +418,7 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	case 1:
 	  /* No op */
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("NOP"), FALSE);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("NOP"), FALSE);
 	  opts_done++;
 	  opts_p++;
 	  break;
@@ -542,17 +433,17 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	  if (opts_done > opts_len)
 	    break;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("Sec"), is_error);
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("Sec"), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	  opts_p++;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[2], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[2], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
 	  opts_p += 2;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[3], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[3], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
 	  opts_p += 2;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[4], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[4], DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
 	  opts_p += 2;
 	  
 #ifdef WORDS_BIGENDIAN
@@ -560,7 +451,7 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 #else
 	  val = (opts_p[0] << 16 | opts_p[1] << 8 | opts_p[2]);
 #endif
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[5], DATA_TO_PTR(val), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[5], DATA_TO_PTR(val), is_error);
 	  opts_p += 3;
 	  break;
 	  
@@ -574,16 +465,16 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	  if (opts_done > opts_len)
 	    break;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("LS&RR"), is_error);
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("LS&RR"), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	  opts_p++;
 	  
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
 	  opts_p++;
 	  
 	  while (opts_p < o_next)
 	    {
-	      nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[7],
+	      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[7],
 				     inet_ntoa(*((struct in_addr *)opts_p)), is_error);
 	      opts_p += 4;
 	    }
@@ -599,16 +490,16 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	  if (opts_done > opts_len)
 	    break;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("SS&RR"), is_error);
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("SS&RR"), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	  opts_p++;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
 	  opts_p++;
 
 	  while (opts_p < o_next)
 	    {
-	      nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[7],
+	      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[7],
 				     inet_ntoa(*((struct in_addr *)opts_p)), is_error);
 	      opts_p += 4;
 	    }
@@ -624,16 +515,16 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	  if (opts_done > opts_len)
 	    break;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("RR"), is_error);
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("RR"), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	  opts_p++;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[6], DATA_TO_PTR(*opts_p), is_error);
 	  opts_p++;
 	  
 	  while (opts_p < o_next)
 	    {
-	      nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[7],
+	      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[7],
 				     inet_ntoa(*((struct in_addr *)opts_p)), is_error);
 	      opts_p += 4;
 	    }
@@ -648,11 +539,11 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	  if (opts_done > opts_len)
 	    break;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("SID"), is_error);
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("SID"), is_error);
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	  opts_p++;
 
-	  nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[8],
+	  nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[8],
 				 DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
 	  opts_p++;
 	  break;
@@ -670,15 +561,15 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	    if (opts_done > opts_len)
 	      break;
 
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("TS"), is_error);
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("TS"), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	    opts_p++;
 		
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[9],
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[9],
 				   DATA_TO_PTR((*opts_p & 0xF0) >> 4), is_error);
 	    
 	    flg = (*opts_p & 0x0F);
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[10], DATA_TO_PTR(flg), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[10], DATA_TO_PTR(flg), is_error);
 	    opts_p++;
 		
 	    while (opts_p < o_next)
@@ -693,12 +584,12 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 
 		if (flg != 0)
 		  {
-		    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[7],
+		    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[7],
 					   inet_ntoa(*((struct in_addr *)opts_p)), is_error);
 		    opts_p += 4;
 		  }
 
-		nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[11], time_string, is_error);
+		nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[11], time_string, is_error);
 		opts_p += 4;
 	      }
 	  }
@@ -714,11 +605,11 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	    if (opts_done > opts_len)
 	      break;
 
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], _("RA"), is_error);
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], _("RA"), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	    opts_p++;
 
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[13],
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[13],
 				   DATA_TO_PTR(ntohs(*((guint16 *) opts_p))), is_error);
 	    opts_p += 2;
 	    break;
@@ -736,12 +627,12 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 	    if (opts_done > opts_len)
 	      break;
 	    
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[0], s, is_error);
-	    nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[0], s, is_error);
+	    nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[1], DATA_TO_PTR(o_len), is_error);
 	    opts_p++;
 	    
 	    if (o_len > 2)
-	      nd_gui_proto_table_add(packet->trace, pinf, &ip_opt_fields[12], DATA_TO_PTR(o_len - 2), is_error);
+	      nd_gui_proto_table_add(libnd_packet_get_trace(packet), pinf, &ip_opt_fields[12], DATA_TO_PTR(o_len - 2), is_error);
 	    
 	    opts_p += o_len;
 	  }
@@ -753,30 +644,17 @@ nd_ip_set_gui_options(ND_ProtoInfo *pinf, struct ip *iphdr,
 }
 
 
-inline ND_Protocol *
+inline LND_Protocol *
 nd_ip_get(void)
 {
   return ip;
 }
 
 
-gboolean   
-nd_ip_csum_correct(const struct ip *iphdr, guint16 *correct_sum)
+inline ND_Protocol *
+nd_ip_get_gui(void)
 {
-  guint16 old_sum, correct_sum_tmp;
-
-  if (!iphdr)
-    return FALSE;
-
-  old_sum = iphdr->ip_sum;
-  ((struct ip *) iphdr)->ip_sum = 0;
-
-  correct_sum_tmp = nd_misc_in_cksum((u_short *)iphdr, iphdr->ip_hl << 2, 0);
-
-  ((struct ip *) iphdr)->ip_sum = old_sum;
-
-  if (correct_sum)
-    *correct_sum = correct_sum_tmp;
-
-  return (old_sum == correct_sum_tmp);
+  return ip_gui;
 }
+
+
