@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2003 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -38,13 +38,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <netinet/in.h>
 #include <netinet/ip.h>
 
-#include <netdude/nd.h>
-#include <netdude/nd_globals.h>
-#include <netdude/nd_debug.h>
-#include <netdude/nd_dialog.h>
-#include <netdude/nd_macros.h>
-#include <netdude/nd_trace.h>
-#include <netdude/nd_packet_iterator.h>
+#include <nd.h>
+#include <nd_dialog.h>
+#include <nd_trace.h>
+#include <nd_trace_registry.h>
 #include <nd_ip.h>
 #include <nd_ipfrag.h>
 #include <interface.h>
@@ -52,7 +49,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <support.h>
 
 void      
-nd_ip_frag_show_dialog(ND_Packet *packet)
+nd_ip_frag_show_dialog(LND_Packet *packet)
 {
   ND_FragData     *data;
   struct ip       *iphdr;
@@ -64,7 +61,7 @@ nd_ip_frag_show_dialog(ND_Packet *packet)
   if (!packet)
     return;
 
-  iphdr = (struct ip *) nd_packet_get_data(packet, nd_ip_get(), 0);
+  iphdr = (struct ip *) libnd_packet_get_data(packet, nd_ip_get(), 0);
   if (!iphdr)
     return;
 
@@ -80,10 +77,8 @@ nd_ip_frag_show_dialog(ND_Packet *packet)
   data->dialog = create_ip_frag_dialog();
 
   D_ASSERT_PTR(data->dialog);
-  sb1 = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag1_spinbutton");
-  D_ASSERT_PTR(sb1);
-  sb2 = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag2_spinbutton");
-  D_ASSERT_PTR(sb1);
+  ND_GTK_GET(sb1, data->dialog, "ip_frag1_spinbutton");
+  ND_GTK_GET(sb2, data->dialog, "ip_frag2_spinbutton");
 
   gtk_object_set_data_full(GTK_OBJECT(data->dialog), "data", data,
 			   (GtkDestroyNotify) g_free);
@@ -111,13 +106,11 @@ nd_ip_frag_show_dialog(ND_Packet *packet)
   data->offset2 = data->offset1 + data->size1;
 
   D_ASSERT_PTR(data->dialog);
-  label = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag1_offset_label");
-  D_ASSERT_PTR(label);
+  ND_GTK_GET(label, data->dialog, "ip_frag1_offset_label");
   g_snprintf(s, 128, "%u", data->offset1);
   gtk_label_set_text(label, s);
 
-  label = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag2_offset_label");
-  D_ASSERT_PTR(label);
+  ND_GTK_GET(label, data->dialog, "ip_frag2_offset_label");
   g_snprintf(s, 128, "%u", data->offset2);
   gtk_label_set_text(label, s);
 
@@ -154,10 +147,8 @@ nd_ip_frag_adjust(ND_FragData *data, int fragment_size, int fragment_number)
   
   data->offset2 = data->offset1 + data->size1;
 
-  sb1 = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag1_spinbutton");
-  D_ASSERT_PTR(sb1);
-  sb2 = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag2_spinbutton");
-  D_ASSERT_PTR(sb2);
+  ND_GTK_GET(sb1, data->dialog, "ip_frag1_spinbutton");
+  ND_GTK_GET(sb2, data->dialog, "ip_frag2_spinbutton");
 
   /* Disable callbacks for a moment to not trigger callbacks */
   gtk_signal_handler_block_by_func(GTK_OBJECT(sb1), on_ip_frag1_spinbutton_changed, NULL);
@@ -172,8 +163,7 @@ nd_ip_frag_adjust(ND_FragData *data, int fragment_size, int fragment_number)
   gtk_signal_handler_unblock_by_func(GTK_OBJECT(sb1), on_ip_frag1_spinbutton_changed, NULL);
   gtk_signal_handler_unblock_by_func(GTK_OBJECT(sb2), on_ip_frag2_spinbutton_changed, NULL);
 
-  label = gtk_object_get_data(GTK_OBJECT(data->dialog), "ip_frag2_offset_label");
-  D_ASSERT_PTR(label);
+  ND_GTK_GET(label, data->dialog, "ip_frag2_offset_label");
   g_snprintf(s, 128, "%u", data->offset2);
   gtk_label_set_text(label, s);
 }
@@ -201,13 +191,13 @@ nd_ip_frag_apply(ND_FragData *data)
 
 
 void      
-nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
+nd_ip_frag_fragment(LND_Packet *packet, guint size1, guint size2)
 {
-  ND_Protocol *ip;
-  struct ip   *iphdr;
-  ND_Packet   *p_copy;
-  guchar      *payload_start, *payload_end, *fragment_start, *fragment_end;
-  int          remainder_len, offset;
+  LND_Protocol  *ip;
+  struct ip     *iphdr;
+  LND_Packet    *p_copy;
+  guchar        *payload_start, *payload_end, *fragment_start, *fragment_end;
+  int            remainder_len, offset;
 
   if (!packet)
     return;
@@ -216,8 +206,8 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
 
   /* Create the second fragment: */
 
-  p_copy = nd_packet_duplicate(packet);
-  iphdr = (struct ip *) nd_packet_get_data(p_copy, ip, 0);
+  p_copy = libnd_packet_duplicate(packet);
+  iphdr = (struct ip *) libnd_packet_get_data(p_copy, ip, 0);
   D_ASSERT_PTR(iphdr);
 
   if (!iphdr)
@@ -244,9 +234,9 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
 
   offset         = (ntohs(iphdr->ip_off) & IP_OFFMASK) * 8 + size1;
   payload_start  = ((guchar *) iphdr) + (iphdr->ip_hl * 4);
-  payload_end    = nd_packet_get_data_end(p_copy, ip, 0);
+  payload_end    = libnd_packet_get_data_end(p_copy, ip, 0);
   fragment_start = payload_start + size1;  
-  remainder_len  = nd_packet_get_end(p_copy) - fragment_start;
+  remainder_len  = libnd_packet_get_end(p_copy) - fragment_start;
 
   p_copy->ph.caplen -= size1;
   p_copy->ph.len -= size1;
@@ -264,7 +254,7 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
   iphdr->ip_len = htons(ntohs(iphdr->ip_len) - size1);
 
   /* .. and fix the checksum. */
-  nd_ip_fix_packet(p_copy, -1);
+  libnd_ip_fix_packet(p_copy);
 
   /* Realloc the packet's memory. All protocol offsets may become
      invalid at this point, but we re-initialize immediately below: */
@@ -272,16 +262,15 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
 
   /* We've just cut out a part of the packet -- reinitialize the
      packet: */
-  nd_packet_init(p_copy);
-
+  libnd_packet_init(p_copy);
 
   /* Now adjust the original packet: */
 
-  iphdr = (struct ip *) nd_packet_get_data(packet, ip, 0);
+  iphdr = (struct ip *) libnd_packet_get_data(packet, ip, 0);
   fragment_start = ((guchar *) iphdr) + (iphdr->ip_hl * 4);
-  payload_end    = nd_packet_get_data_end(packet, ip, 0);
+  payload_end    = libnd_packet_get_data_end(packet, ip, 0);
   fragment_end   = fragment_start + size1;
-  remainder_len  = nd_packet_get_end(packet) - payload_end;
+  remainder_len  = libnd_packet_get_end(packet) - payload_end;
 
   packet->ph.caplen -= size2;
   packet->ph.len -= size2;
@@ -294,15 +283,15 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
   /* Correct IP fields -- fragmentation flags, IP length, checksum: */
   iphdr->ip_off = htons((ntohs(iphdr->ip_off) | IP_MF) & ~IP_DF);
   iphdr->ip_len = htons(ntohs(iphdr->ip_len) - size2);
-  nd_ip_fix_packet(packet, -1);
+  libnd_ip_fix_packet(packet);
 
   /* And reinitialize the whole thing */
-  nd_packet_init(packet);
-  nd_packet_modified(packet);
+  libnd_packet_init(packet);
+  libnd_packet_modified(packet);
 
   /* Insert new fragment after the original into trace: */
-  nd_trace_packet_insert_at_index(packet->trace, p_copy,
-				  nd_packet_get_index(packet) + 1);
+  libnd_tp_insert_packets(packet->part, p_copy,
+			  libnd_packet_get_index(packet) + 1);
 }
 
 
@@ -313,15 +302,15 @@ nd_ip_frag_fragment(ND_Packet *packet, guint size1, guint size2)
 static gint 
 ip_off_cmp(gconstpointer d1, gconstpointer d2)
 {
-  ND_Packet *p1, *p2;
+  LND_Packet *p1, *p2;
   struct ip *iphdr1, *iphdr2;
 
-  p1 = (ND_Packet *) d1;
-  p2 = (ND_Packet *) d2;
+  p1 = (LND_Packet *) d1;
+  p2 = (LND_Packet *) d2;
 
-  iphdr1 = (struct ip *) nd_packet_get_data(p1, nd_ip_get(), 0);
+  iphdr1 = (struct ip *) libnd_packet_get_data(p1, nd_ip_get(), 0);
   D_ASSERT_PTR(iphdr1);
-  iphdr2 = (struct ip *) nd_packet_get_data(p2, nd_ip_get(), 0);
+  iphdr2 = (struct ip *) libnd_packet_get_data(p2, nd_ip_get(), 0);
   D_ASSERT_PTR(iphdr2);
 
   if (ND_IP_FRAG_OFFSET(iphdr1) < ND_IP_FRAG_OFFSET(iphdr2))
@@ -339,14 +328,14 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
   struct ip        *base_iphdr, *frag_iphdr;
   GList            *f, *f2, *f3;
   int               payload_increase, trailer_len, covered_offset;
-  ND_Packet        *base_packet, *frag_packet, *input_packet;
+  LND_Packet       *base_packet, *frag_packet, *input_packet;
   guchar           *trailer, *base_payload, *dest;
   
 
   if (!fraglist || (g_list_length(fraglist) == 1))
     return;
 
-  input_packet = (ND_Packet *) user_data;
+  input_packet = (LND_Packet *) user_data;
 
   D(("Beginning reassembly.\n"));
 
@@ -355,8 +344,8 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 
   for (f = fraglist; f; /* Increment depends on fragments, see below */)
     {
-      base_packet = (ND_Packet *) f->data;
-      base_iphdr = (struct ip *) nd_packet_get_data(base_packet, nd_ip_get(), 0);
+      base_packet = (LND_Packet *) f->data;
+      base_iphdr = (struct ip *) libnd_packet_get_data(base_packet, nd_ip_get(), 0);
       D_ASSERT_PTR(base_iphdr);
 
       covered_offset = ND_IP_FRAG_OFFSET(base_iphdr) + ND_IP_PAYLOAD_LENGTH(base_iphdr);
@@ -367,7 +356,7 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 	
       while (f2)
 	{
-	  frag_iphdr = (struct ip *) nd_packet_get_data((ND_Packet *) f2->data, nd_ip_get(), 0);
+	  frag_iphdr = (struct ip *) libnd_packet_get_data((LND_Packet *) f2->data, nd_ip_get(), 0);
 
 	  if (ND_IP_FRAG_OFFSET(frag_iphdr) > covered_offset)
 	    break;
@@ -399,12 +388,12 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 		base_packet->ph.caplen);
       
       /* This may have made offsets invalid, so re-init: */
-      nd_packet_init(base_packet);
-      base_iphdr = (struct ip *) nd_packet_get_data(base_packet, nd_ip_get(), 0);
+      libnd_packet_init(base_packet);
+      base_iphdr = (struct ip *) libnd_packet_get_data(base_packet, nd_ip_get(), 0);
 
       base_payload = ND_IP_PAYLOAD(base_iphdr);
-      trailer = nd_packet_get_data_end(base_packet, nd_ip_get(), 0);
-      trailer_len = nd_packet_get_end(base_packet) - trailer;
+      trailer = libnd_packet_get_data_end(base_packet, nd_ip_get(), 0);
+      trailer_len = libnd_packet_get_end(base_packet) - trailer;
 
       D(("Reallocd content, adjusted length values.\n"));
 
@@ -419,8 +408,8 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 	 the first packet since its payload is already there) */
       for (f3 = g_list_next(f); f3 && (f3 != f2) ; f3 = g_list_next(f3))
 	{
-	  frag_packet = (ND_Packet *) f3->data;
-	  frag_iphdr  = (struct ip *) nd_packet_get_data(frag_packet, nd_ip_get(), 0);
+	  frag_packet = (LND_Packet *) f3->data;
+	  frag_iphdr  = (struct ip *) libnd_packet_get_data(frag_packet, nd_ip_get(), 0);
 
 	  D(("Merging one packet.\n"));
 	  
@@ -445,15 +434,16 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 	  
 	  /* This fragment is reassembled and can thus be nuked: */
 	  if (input_packet == frag_packet)
-	    nd_trace_set_current_packet(base_packet->trace, NULL);
+	    nd_trace_set_current_packet(libnd_packet_get_trace(base_packet), NULL);
 	  
-	  nd_packet_delete(frag_packet, TRUE);
+	  libnd_packet_remove(frag_packet);
+	  libnd_packet_free(frag_packet);
 	  f3->data = NULL;
 	}
 
       D(("Fixing checksum in merged packet.\n"));
-      nd_ip_fix_packet(base_packet, -1);
-      nd_packet_modified(base_packet);
+      libnd_ip_fix_packet(base_packet);
+      libnd_packet_modified(base_packet);
 
       /* If this is a now-complete IP packet, reinitialize it
 	 so that payload protocols become visible: */
@@ -461,7 +451,7 @@ ip_frag_reassemble_packet(gpointer key, GList *fraglist, gpointer user_data)
 	  ((ntohs(base_iphdr->ip_off) & IP_MF) == 0))
 	{
 	  D(("New packet is complete IP datagram, initializing ...\n"));
-	  nd_packet_init(base_packet);
+	  libnd_packet_init(base_packet);
 	}
 
       /* Continue where new consecutive fragments start: */
@@ -486,29 +476,41 @@ ip_frag_free(gpointer key, GList *fraglist, gpointer user_data)
 
 
 void      
-nd_ip_frag_reassemble(ND_Packet *input_packet)
+nd_ip_frag_reassemble(LND_Packet *input_packet)
 {
-  struct ip          *iphdr;
-  ND_Packet          *packet;
-  ND_Trace           *trace;
-  GHashTable         *fraggles;
-  GList              *fragslist = NULL;
-  ND_PacketIterator   pit;
-  int                 fragments = 0, packets = 0;
-  char                message[MAXPATHLEN];
+  struct ip               *iphdr;
+  LND_Packet              *packet;
+  LND_Trace               *trace;
+  GHashTable              *fraggles;
+  GList                   *fragslist = NULL;
+  LND_PacketIterator       pit;
+  LND_PacketIteratorMode   mode;
+  int                      fragments = 0, packets = 0;
+  char                     message[MAXPATHLEN];
 
-  if (!input_packet || !input_packet->trace)
+  if (!input_packet || !libnd_packet_get_trace(input_packet))
     return_if_no_current_trace(trace);
   else
-    trace = input_packet->trace;
+    trace = libnd_packet_get_trace(input_packet);
 
   fraggles = g_hash_table_new(g_direct_hash, g_direct_equal);
+  mode = trace->iterator_mode;
 
-  for (nd_pit_init(&pit, trace, TRUE); nd_pit_get(&pit); nd_pit_next(&pit))
+  if (mode == LND_PACKET_IT_AREA_R || mode == LND_PACKET_IT_AREA_RW)
     {
-      packet = nd_pit_get(&pit);
+      /* We're currently only reassembling packets in memory,
+       * so reduce this to apply-to-trace-part if the current
+       * mode is apply-to-all:
+       */
 
-      iphdr = (struct ip *) nd_packet_get_data(packet, nd_ip_get(), 0);
+      mode = LND_PACKET_IT_PART_RW;
+    }
+
+  for (libnd_pit_init_mode(&pit, trace, mode); libnd_pit_get(&pit); libnd_pit_next(&pit))
+    {
+      packet = libnd_pit_get(&pit);
+
+      iphdr = (struct ip *) libnd_packet_get_data(packet, nd_ip_get(), 0);
       if (!iphdr)
 	continue;
 
