@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -40,22 +40,24 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <interface.h>
 #include <support.h>
 
-#include <netdude/nd.h>
-#include <netdude/nd_debug.h>
-#include <netdude/nd_capture.h>
-#include <netdude/nd_dialog.h>
-#include <netdude/nd_clipboard.h>
-#include <netdude/nd_globals.h>
-#include <netdude/nd_gui.h>
-#include <netdude/nd_main.h>
-#include <netdude/nd_misc.h>
-#include <netdude/nd_trace.h>
-#include <netdude/nd_trace_registry.h>
-#include <netdude/nd_plugin.h>
-#include <netdude/nd_prefs.h>
-#include <netdude/nd_recent.h>
-#include <netdude/nd_protocol_plugin.h>
-#include <netdude/nd_protocol_inst.h>
+#include <nd.h>
+#include <nd_dialog.h>
+#include <nd_clipboard.h>
+#include <nd_filter_gui.h>
+#include <nd_globals.h>
+#include <nd_gui.h>
+#include <nd_main.h>
+#include <nd_trace.h>
+#include <nd_trace_registry.h>
+#include <nd_tpm_gui.h>
+#include <nd_tp.h>
+#include <nd_plugin.h>
+#include <nd_prefs.h>
+#include <nd_recent.h>
+#include <nd_trace_area_gui.h>
+#include <nd_protocol_plugin.h>
+#include <nd_protocol_inst.h>
+#include <nd_info_dialog.h>
 
 void
 on_filesel_ok_clicked                  (GtkButton       *button,
@@ -148,7 +150,11 @@ on_trace_list_row_move               (GtkCList        *clist,
 				      gint             arg2,
 				      gpointer         user_data)
 {
-  nd_trace_move_packet(nd_trace_registry_get_current(), arg1, arg2);
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+
+  libnd_tp_move_packet(trace->tpm->current, arg1, arg2);
 
   return;
   TOUCH(clist);
@@ -156,30 +162,6 @@ on_trace_list_row_move               (GtkCList        *clist,
 }
 
 
-void
-on_packet_hide_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-  nd_trace_sel_hide(nd_trace_registry_get_current());
-
-  return;
-  TOUCH(menuitem);
-  TOUCH(user_data);
-}
-
-
-void
-on_packet_show_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
-{
-  nd_trace_sel_show(nd_trace_registry_get_current());
-
-  return;
-  TOUCH(menuitem);
-  TOUCH(user_data);
-}
-
-
 void
 on_packet_cut_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -222,7 +204,7 @@ on_trace_list_button_press_event     (GtkWidget       *widget,
 				      GdkEventButton  *event,
 				      gpointer         user_data)
 {
-  ND_Trace     *trace;
+  LND_Trace     *trace;
   int           row, col;
 
   D_ENTER;
@@ -230,17 +212,16 @@ on_trace_list_button_press_event     (GtkWidget       *widget,
   return_val_if_no_current_trace(trace, FALSE);
   nd_gui_timestamp_hide();
 
-  if ((event->button == 1)                      &&
+  if ((event->button == 1) &&
       ((event->state & GDK_MODIFIER_MASK) == 0))
     {
-      nd_trace_clear_selection(trace);
-      gtk_clist_unselect_all(GTK_CLIST(widget));
+      libnd_tp_clear_selection(trace->tpm->current);
     }
   else if (event->button == 3)
     {
       gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y, &row, &col);
 
-      if (nd_trace_sel_size(trace) == 0)
+      if (libnd_tp_get_sel_size(trace->tpm->current) == 0)
 	gtk_clist_select_row(GTK_CLIST(widget), row, 0);
 
       /* nd_packet_set(nd_trace_packet_get_nth(row)); */
@@ -248,7 +229,7 @@ on_trace_list_button_press_event     (GtkWidget       *widget,
       nd_gui_show_packet_menu(event);
     }
 
-  D_RETURN_(FALSE);
+  D_RETURN_(TRUE);
   TOUCH(user_data);
 }
 
@@ -260,10 +241,15 @@ on_trace_list_unselect_row           (GtkCList        *clist,
 				      GdkEvent        *event,
 				      gpointer         user_data)
 {
-  D_ENTER;
-  nd_trace_unselect_packet(nd_trace_registry_get_current(), row);
+  LND_Trace *trace;
 
+  return_if_no_current_trace(trace);
+  
+
+  D_ENTER;
+  libnd_tp_unselect_packet(trace->tpm->current, row);
   D_RETURN;
+
   TOUCH(user_data);
   TOUCH(clist);
   TOUCH(event);
@@ -278,12 +264,12 @@ on_trace_list_select_row             (GtkCList        *clist,
 				      GdkEvent        *event,
 				      gpointer         user_data)
 {
-  ND_Trace *trace;
+  LND_Trace *trace;
 
   D_ENTER;
   return_if_no_current_trace(trace);
 
-  nd_trace_select_packet(trace, row);
+  nd_tp_select_packet(trace->tpm->current, row);
   
   D_RETURN;
   TOUCH(user_data);
@@ -296,17 +282,15 @@ void
 on_packet_unselect_all_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  ND_Trace   *trace;
+  LND_Trace   *trace;
 
   D_ENTER;
   return_if_no_current_trace(trace);
-  gtk_clist_unselect_all(GTK_CLIST(trace->list));
 
   /* This called OUR callback -- no signals are sent,
    * so we need to do this manually
    */
-  nd_trace_clear_selection(trace);
-  nd_trace_set_current_packet(trace, NULL);
+  libnd_tp_clear_selection(trace->tpm->current);
 
   D_RETURN;
   TOUCH(user_data);
@@ -318,19 +302,12 @@ void
 on_packet_select_all_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  ND_Trace   *trace;
+  LND_Trace   *trace;
 
   D_ENTER;
   return_if_no_current_trace(trace);
-  gtk_clist_select_all(GTK_CLIST(trace->list));
 
-  /* This called OUR callback -- no signals are sent,
-   * so we need to do this manually
-   */
-    
-  nd_trace_full_selection(trace);
-  nd_trace_set_current_packet(trace,
-			      nd_trace_packet_get_nth(trace, 0));
+  libnd_tp_full_selection(trace->tpm->current);
 
   D_RETURN;
   TOUCH(user_data);
@@ -351,40 +328,32 @@ void
 on_packet_delete_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  D_ENTER;
-  nd_trace_sel_delete(nd_trace_registry_get_current());
+  LND_Trace *trace;
 
+  return_if_no_current_trace(trace);
+
+  D_ENTER;
+  libnd_tp_sel_delete(trace->tpm->current);
   D_RETURN;
+
   TOUCH(user_data);
   TOUCH(menuitem);
 }
 
 
-gboolean
-on_menubar_button_press_event          (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data)
-{
-  nd_gui_sync();
-  
-  return FALSE;
-  TOUCH(widget);
-  TOUCH(event);
-  TOUCH(user_data);
-}
-
-
 void
 on_trace_list_select_all             (GtkCList        *clist,
 				      gpointer         user_data)
 {
-  ND_Trace *trace;
+  LND_Trace *trace;
 
   D_ENTER;
   return_if_no_current_trace(trace);
 
-  nd_trace_full_selection(trace);
-  nd_trace_set_current_packet(trace, nd_trace_packet_get_nth(trace, 0));
+  /* This one is called whenever the GTK clist fires a select-all
+   * event, therefore we only need to inform libnetdude:
+   */
+  libnd_tp_full_selection(trace->tpm->current);
 
   D_RETURN;
   TOUCH(clist);
@@ -396,10 +365,14 @@ void
 on_trace_list_unselect_all           (GtkCList        *clist,
 				      gpointer         user_data)
 {
-  D_ENTER;
-  nd_trace_clear_selection(nd_trace_registry_get_current());
+  LND_Trace *trace;
 
+  return_if_no_current_trace(trace);
+
+  D_ENTER;
+  libnd_tp_clear_selection(trace->tpm->current);
   D_RETURN;
+
   TOUCH(clist);
   TOUCH(user_data);
 }
@@ -493,18 +466,19 @@ on_trace_win_bar_button_release_event  (GtkWidget       *widget,
 
 
 void
-on_plugin_activate                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_plugin_activate                     (gpointer         user_data,
+					gint             action,
+					GtkWidget       *widget)
 {
-  ND_Plugin *plugin;
+  ND_Plugin *plugin = (ND_Plugin *) user_data;
 
-  plugin = (ND_Plugin*) gtk_object_get_data(GTK_OBJECT(menuitem), "plugin");
   D_ASSERT_PTR(plugin);
-
   nd_plugin_run(plugin);
 
   return;
   TOUCH(user_data);
+  TOUCH(action);
+  TOUCH(widget);
 }
 
 
@@ -649,16 +623,19 @@ on_recent_item4_activate               (GtkMenuItem     *menuitem,
 
 
 void
-on_about_plugin_activate               (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_about_plugin_activate               (gpointer         user_data,
+					gint             action,
+					GtkWidget       *widget)
 {
-  ND_Plugin *plugin;
+  ND_Plugin *plugin = (ND_Plugin *) user_data;
 
-  plugin = (ND_Plugin*) gtk_object_get_data(GTK_OBJECT(menuitem), "plugin");
+  D_ASSERT_PTR(plugin);
   nd_plugin_show_about(plugin);
 
   return;
   TOUCH(user_data);
+  TOUCH(action);
+  TOUCH(widget);
 }
 
 
@@ -680,7 +657,7 @@ void
 on_trace_close_clicked                 (GtkButton       *button,
                                         gpointer         user_data)
 {
-  ND_Trace *trace = (ND_Trace *) user_data;
+  LND_Trace *trace = (LND_Trace *) user_data;
 
   nd_trace_close(trace);
 
@@ -689,13 +666,27 @@ on_trace_close_clicked                 (GtkButton       *button,
 }
 
 
+
+gboolean
+on_trace_menu_press_event              (GtkWidget       *widget,
+					GdkEventButton  *event,
+					gpointer         user_data)
+{
+  nd_gui_show_iterator_mode_menu(event);
+
+  return TRUE;
+  TOUCH(user_data);
+  TOUCH(widget);
+}
+
+
 void
 on_traces_notebook_switch_page         (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
                                         gint             page_num,
                                         gpointer         user_data)
 {
-  ND_Trace *trace;
+  LND_Trace *trace;
 
   D_ENTER;
   trace = gtk_object_get_data(GTK_OBJECT(page->child), "trace");
@@ -716,14 +707,18 @@ on_protos_notebook_switch_page         (GtkNotebook     *notebook,
                                         gint             page_num,
                                         gpointer         user_data)
 {
-  ND_Trace        *trace;
-  ND_ProtoInfo    *pinf;
-  ND_ProtoData    *pd;
+  LND_Trace        *trace;
+  ND_Trace         *trace_gui;
+  LND_ProtoInfo    *pinf;
+  LND_ProtoData    *pd;
 
   trace = gtk_object_get_data(GTK_OBJECT(notebook), "trace");
-  D_ASSERT_PTR(trace);
+  trace_gui = nd_trace_get(trace);
 
-  if (!trace->cur_packet)
+  if (!trace_gui)
+    return;
+
+  if (!trace_gui->cur_packet)
     return;
 
   pinf = gtk_object_get_data(GTK_OBJECT(page->child), "pinf");
@@ -732,7 +727,7 @@ on_protos_notebook_switch_page         (GtkNotebook     *notebook,
   if (!pinf)
     return;
 
-  pd = nd_packet_get_proto_data(trace->cur_packet, pinf->inst.proto, pinf->inst.nesting);
+  pd = libnd_packet_get_proto_data(trace_gui->cur_packet, pinf->inst.proto, pinf->inst.nesting);
   nd_trace_set_current_proto_selection(trace, &pd->inst);
   nd_trace_set_current_proto(trace, &pd->inst, TRUE);
   
@@ -758,7 +753,7 @@ void
 on_file_save_as_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  ND_Trace *trace;
+  LND_Trace *trace;
 
   return_if_no_current_trace(trace);
   nd_trace_save_as_dialog(trace, NULL, NULL);
@@ -855,10 +850,10 @@ void
 on_font_ok_button_clicked              (GtkButton       *button,
                                         gpointer         user_data)
 {
-  GtkWidget *toplevel;
+  GtkWidget *nd_toplevel_window;
 
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  nd_dialog_fontsel_ok(GTK_FONT_SELECTION_DIALOG(toplevel));
+  nd_toplevel_window = gtk_widget_get_toplevel(GTK_WIDGET(button));
+  nd_dialog_fontsel_ok(GTK_FONT_SELECTION_DIALOG(nd_toplevel_window));
 
   return;
   TOUCH(user_data);
@@ -869,10 +864,10 @@ void
 on_font_cancel_button_clicked          (GtkButton       *button,
                                         gpointer         user_data)
 {
-  GtkWidget *toplevel;
+  GtkWidget *nd_toplevel_window;
 
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  nd_dialog_fontsel_cancel(GTK_FONT_SELECTION_DIALOG(toplevel));
+  nd_toplevel_window = gtk_widget_get_toplevel(GTK_WIDGET(button));
+  nd_dialog_fontsel_cancel(GTK_FONT_SELECTION_DIALOG(nd_toplevel_window));
 
   return;
   TOUCH(user_data);
@@ -883,10 +878,10 @@ void
 on_font_apply_button_clicked           (GtkButton       *button,
                                         gpointer         user_data)
 {
-  GtkWidget *toplevel;
+  GtkWidget *nd_toplevel_window;
 
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  nd_dialog_fontsel_apply(GTK_FONT_SELECTION_DIALOG(toplevel));
+  nd_toplevel_window = gtk_widget_get_toplevel(GTK_WIDGET(button));
+  nd_dialog_fontsel_apply(GTK_FONT_SELECTION_DIALOG(nd_toplevel_window));
 
   return;
   TOUCH(user_data);
@@ -1071,10 +1066,10 @@ on_largenumber_down_clicked             (GtkButton       *button,
 
 
 void
-on_capt_start_clicked                  (GtkButton       *button,
+on_string_ok_button_clicked            (GtkButton       *button,
                                         gpointer         user_data)
 {
-  nd_capture_dialog_start(gtk_widget_get_toplevel(GTK_WIDGET(button)));
+  nd_dialog_string_ok(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 
   return;
   TOUCH(user_data);
@@ -1082,150 +1077,33 @@ on_capt_start_clicked                  (GtkButton       *button,
 
 
 void
-on_capt_close_clicked                  (GtkButton       *button,
+on_string_cancel_button_clicked        (GtkButton       *button,
                                         gpointer         user_data)
 {
-  ND_Capture *capt;
-  GtkWidget *toplevel;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  capt = gtk_object_get_data(GTK_OBJECT(toplevel), "capture");
-  nd_capture_stop(capt);
-  nd_capture_dialog_close(gtk_widget_get_toplevel(GTK_WIDGET(button)));
+  nd_dialog_string_cancel(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 
   return;
   TOUCH(user_data);
 }
 
 
-void
-on_capt_size_lim_checkbutton_toggled   (GtkToggleButton *togglebutton,
+gboolean
+on_string_entry_key_press_event        (GtkWidget       *widget,
+                                        GdkEventKey     *event,
                                         gpointer         user_data)
 {
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_size_spinbutton");
-  D_ASSERT_PTR(w);
-  gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
+  if (event->keyval == GDK_Return)
+    nd_dialog_string_ok(gtk_widget_get_toplevel(widget));
 
-  return;
+  return FALSE;
   TOUCH(user_data);
 }
 
 
 void
-on_capt_stop_count_checkbutton_toggled (GtkToggleButton *togglebutton,
+on_pref_tmpdir_browse_button_clicked   (GtkButton       *button,
                                         gpointer         user_data)
 {
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_stop_count_spinbutton");
-  D_ASSERT_PTR(w);
-  gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
-
-  return;
-  TOUCH(user_data);
-}
-
-
-void
-on_capt_stop_time_checkbutton_toggled  (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_stop_time_spinbutton");
-  D_ASSERT_PTR(w);
-  gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
-
-  return;
-  TOUCH(user_data);
-}
-
-
-void
-on_capt_stop_size_checkbutton_toggled  (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_stop_size_spinbutton");
-  D_ASSERT_PTR(w);
-  gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
-
-  return;
-  TOUCH(user_data);
-}
-
-
-void
-on_capt_lim_checkbutton_toggled        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_limit_spinbutton");
-  D_ASSERT_PTR(w);
-  gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
-
-  return;
-  TOUCH(user_data);
-}
-
-
-void
-on_capt_filter_file_checkbutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  GtkWidget *toplevel, *w;
-  
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
-
-  if (gtk_toggle_button_get_active(togglebutton))
-    {
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_entry");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, FALSE);
-
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_file_combo");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, TRUE);
-
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_file_button");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, TRUE);
-    }
-  else
-    {
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_entry");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, TRUE);
-
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_file_combo");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, FALSE);
-
-      w = gtk_object_get_data(GTK_OBJECT(toplevel), "capt_filter_file_button");
-      D_ASSERT_PTR(w);
-      gtk_widget_set_sensitive(w, FALSE);
-    }
-
-  return;
-  TOUCH(user_data);
-}
-
-
-void
-on_capt_filter_file_button_clicked     (GtkButton       *button,
-                                        gpointer         user_data)
-{
-  nd_capture_choose_filter_file(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 
   return;
   TOUCH(button);
@@ -1234,10 +1112,64 @@ on_capt_filter_file_button_clicked     (GtkButton       *button,
 
 
 void
-on_capture_activate                    (GtkMenuItem     *menuitem,
+on_reload_plugins_activate             (gpointer         user_data,
+					gint             action,
+					GtkWidget       *widget)
+{
+  nd_plugin_init();
+
+  return;
+  TOUCH(user_data);
+  TOUCH(action);
+  TOUCH(widget);
+}
+
+
+void
+on_apply_to_selection_activate         (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  nd_capture_dialog_show();
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+
+  if (GTK_CHECK_MENU_ITEM(menuitem)->active)
+    {
+      trace->iterator_mode = LND_PACKET_IT_SEL_RW;
+      nd_gui_iterator_mode_set(trace);
+      D(("Switching to selection-only mode.\n"));
+    }
+  
+  return;
+  TOUCH(user_data);
+}
+
+
+void
+on_apply_to_trace_area_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+
+  if (GTK_CHECK_MENU_ITEM(menuitem)->active)
+    {
+      trace->iterator_mode = LND_PACKET_IT_AREA_RW;
+      nd_gui_iterator_mode_set(trace);
+      D(("Switching to area-rw mode.\n"));
+    }
+
+  return;
+  TOUCH(user_data);
+}
+
+
+void
+on_other_trace_area_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_tpm_dialog_show();
 
   return;
   TOUCH(menuitem);
@@ -1246,36 +1178,651 @@ on_capture_activate                    (GtkMenuItem     *menuitem,
 
 
 void
-on_capt_stop_button_clicked            (GtkButton       *button,
+on_nav_load_button_clicked             (GtkButton       *button,
                                         gpointer         user_data)
 {
-  ND_Capture *capt;
-  GtkWidget *toplevel;
+  nd_tpm_dialog_apply();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_nav_cancel_button_clicked           (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_tpm_dialog_hide();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_nav_type_radiobutton_toggled        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+  LND_TPM_NavMode mode;
+
+  mode = nd_tpm_dialog_get_mode();
+  nd_tpm_dialog_set_mode(mode);
+
+  return;
+  TOUCH(togglebutton);
+  TOUCH(user_data);
+}
+
+
+void
+on_nav_time_entries_changed            (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+  nd_tpm_dialog_sync_time_scale();
+
+  return;
+  TOUCH(editable);
+  TOUCH(user_data);
+}
+
+
+void
+on_nav_prev_button_clicked             (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_tpm_dialog_prev_part();
   
-  toplevel = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  capt = gtk_object_get_data(GTK_OBJECT(toplevel), "capture");
-  D_ASSERT_PTR(capt);
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
 
-  nd_capture_stop(capt);
 
+void
+on_nav_next_button_clicked             (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_tpm_dialog_next_part();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_file_info_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_info_dialog_show(trace);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_file_info_close_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_info_dialog_hide();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_filters_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_filter_dialog_show();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_filter_create_button_clicked        (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_filter_dialog_create();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_filter_ok_button_clicked            (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_filter_dialog_hide();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_filter_modify_button_clicked        (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_filter_dialog_modify();
+ 
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_filter_delete_button_clicked        (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_filter_dialog_delete();
+ 
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_filter_update_button_clicked  (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_filter_dialog_apply_filters();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_filter_cancel_button_clicked  (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_filter_dialog_cancel();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_filtering_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_trace_filter_dialog_show();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_next_packet_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_NEXT);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_previous_packet_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_PREV);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_next_with_same_protocol_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_NEXT_PROT);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_previous_with_same_protocol_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_PREV_PROT);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_next_selected_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_NEXT_SEL);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_previous_selected_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_PREV_SEL);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_file_menu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_gui_sync_file_menu();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_edit_menu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_gui_sync_edit_menu();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+
+void
+on_go_menu_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_gui_sync_go_menu();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_protocol_menu_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_gui_sync_protocol_menu();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+void
+on_view_fixed_button_clicked           (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_tpm_dialog_show();
+
+  return;
+  TOUCH(button); TOUCH(user_data);
+}
+
+
+gboolean
+on_view_fixed_button_press_event       (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data)
+{
+  nd_tpm_dialog_show();
+
+  return FALSE;
+  TOUCH(widget); TOUCH(event); TOUCH(user_data);
+}
+
+
+void
+on_packet_filter_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  libnd_tp_sel_filter(trace->tpm->current);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_packet_unfilter_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  libnd_tp_sel_unfilter(trace->tpm->current);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_filtered_delete_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  /* FIXME libnd_trace_delete_hidden_packets(trace); */
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_next_unfiltered_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_NEXT_UNFILTERED);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_previous_unfiltered_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_goto(trace, ND_TRACE_JUMP_PREV_UNFILTERED);
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+gboolean
+on_ip_button_release_event             (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data)
+{
+  if (GTK_SPIN_BUTTON(widget)->in_child)
+    gtk_entry_select_region(GTK_ENTRY(widget), 0, -1);
+
+  return FALSE;
+  TOUCH(event); TOUCH(user_data);
+}
+
+
+void
+on_pit_time_entries_changed            (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_sync_time_scale((gboolean) GPOINTER_TO_INT(user_data));
+
+  return;
+  TOUCH(editable);
+  TOUCH(user_data);
+}
+
+
+void
+on_view_fixed_area_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_show();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_new_button_clicked       (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_new_area();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_del_button_clicked       (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_del_area();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_set_button_clicked       (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_set_area();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_part_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_set_current_part();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_current_button_clicked   (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  LND_Trace *trace;
+
+  return_if_no_current_trace(trace);
+  nd_trace_area_dialog_sync(&trace->area);
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_trace_button_clicked     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_sync(NULL);
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_apply_button_clicked     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_apply();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_trace_area_close_button_clicked     (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_hide();
+
+  return;
+  TOUCH(button);
+  TOUCH(user_data);
+}
+
+
+void
+on_area_list_select                    (GtkList         *list,
+                                        GtkWidget       *widget,
+                                        gpointer         user_data)
+{
+  LND_TraceArea *area;
+
+  ND_GTK_GET(area, widget, "trace_area");
+  if (area)
+    nd_trace_area_dialog_select_area(area);
+
+  return;
+  TOUCH(list);
+  TOUCH(user_data);
+}
+
+
+
+void
+on_trace_areas_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_trace_area_dialog_show();
+
+  return;
+  TOUCH(menuitem);
+  TOUCH(user_data);
+}
+
+
+void
+on_netdude_output_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  nd_runtime_options.debug = GTK_CHECK_MENU_ITEM(menuitem)->active;
+  return;
+  TOUCH(user_data);
+}
+
+
+void
+on_libnetdude_output_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  libnd_runtime_options.debug = GTK_CHECK_MENU_ITEM(menuitem)->active;
+  return;
+  TOUCH(user_data);  
+}
+
+
+void
+on_pcapnav_output_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  pcapnav_runtime_options.debug = GTK_CHECK_MENU_ITEM(menuitem)->active;  
   return;
   TOUCH(user_data);
 }
 
 void
-on_apply_to_all_togglebutton_toggled   (GtkToggleButton *togglebutton,
+on_file_close_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  ND_Trace *trace;
+  LND_Trace *trace;
 
   return_if_no_current_trace(trace);
-
-  if (gtk_toggle_button_get_active(togglebutton))
-    trace->apply_to_all = TRUE;
-  else
-    trace->apply_to_all = FALSE;
+  nd_trace_close(trace);
   
   return;
+  TOUCH(menuitem);
   TOUCH(user_data);
 }
 
