@@ -1,7 +1,6 @@
-
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -28,13 +27,14 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #endif
 
 #include <ctype.h>
+#include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <netdude/nd_macros.h>
-#include <netdude/nd_debug.h>
-#include <netdude/support.h>
-#include <netdude/gtkhex.h>
+#include <nd_macros.h>
+#include <nd_debug.h>
+#include <support.h>
+#include <gtkhex.h>
 
 enum {
   CHANGED,
@@ -47,7 +47,9 @@ static guint hex_signals[LAST_SIGNAL] = { 0 };
 
 struct _GtkHexPrivate
 {
-  GtkText *null_map;
+  gchar   *null_map;
+  int    null_map_len;
+  int    null_map_used;
 };
 
 /* The width of a line of text in the hex-mode view, consisting
@@ -70,6 +72,8 @@ struct _GtkHexPrivate
 #define HEX_LINE_RIGHT_MIDDLE        31
 #define HEX_BLOCK_LEN                23
 
+#define HEX_LINE_BYTES               16
+
 #define NULL_CHAR                    0xF8
 #define NONPRINT_CHAR                0xB7
 
@@ -84,6 +88,37 @@ static void      hex_text_ascii_colorify(GtkHex *hex, guint start, int length);
 static void      hex_text_set_cursor(GtkHex *hex, guint index);
 static gboolean  hex_is_char_unprintable(guchar val);
 
+/* Inserts text into null map */
+static void
+null_map_insert(GtkHexPrivate *priv, gchar *text,
+		int pos, int length)
+{
+  if (priv->null_map_len - priv->null_map_used <= length)
+    {
+      priv->null_map = realloc(priv->null_map, 2 * priv->null_map_len);
+      priv->null_map_len *= 2;
+    }
+  
+  memmove(priv->null_map + pos + length, 
+	  priv->null_map + pos,
+	  priv->null_map_used - pos);
+  memcpy(priv->null_map + pos, text, length);
+  priv->null_map_used += length;
+}
+
+/* Removes text from null map */
+static void
+null_map_delete(GtkHexPrivate *priv, int start, int end)
+{
+  if (end < start)
+    end = priv->null_map_used;
+  
+  memmove(priv->null_map + start,
+	  priv->null_map + end,
+	  priv->null_map_len - end);
+  priv->null_map_used -= (end - start);
+}
+
 
 /* Sets the editor into hex mode
  */
@@ -143,8 +178,7 @@ on_hex_text_deleted(GtkWidget       *widget,
 
   if (hex->mode == GTK_RAW_ASCII)
     {
-      gtk_editable_delete_text(GTK_EDITABLE(hex->priv->null_map),
-			       start, end);
+      null_map_delete(hex->priv, start, end);
       hex_text_ascii_colorify(hex, start, -ABS(end - start));
       hex_sync_data_to_ascii(hex);
 
@@ -182,28 +216,27 @@ on_hex_text_inserted(GtkWidget       *widget,
 	{
 	  memset(whitespace, ' ', length);
 
-	  for (i = 0; i < length + 1; i++)
+	  for (i = 0; i < length; i++)
 	    {
 	      if (hex_is_char_unprintable(text[i]))
 		{
 		  switch (text[i])
 		    {
 		    case '\n':
-		      whitespace[i] = ' ';
+		      whitespace[i] = '\n';
 		      break;
-
+		      
 		    case 0:
 		      whitespace[i] = '0';
 		      break;
-
+		      
 		    default:
 		      whitespace[i] = text[i];
 		    }	      
 		}
 	    }
 	  
-	  gtk_editable_insert_text(GTK_EDITABLE(hex->priv->null_map),
-				   whitespace, length, &pos);
+	  null_map_insert(hex->priv, whitespace, pos, length);
 	  g_free(whitespace);
 	}
     }
@@ -716,6 +749,9 @@ hex_text_data_key_press(GtkHex *hex, GdkEventKey *event, int byte_offset)
 	}
       else if (hex_cursor_over_hex_ascii(hex))
 	{
+	  if (event->string[0] < 32 || event->string[0] > 126)
+	    return FALSE;
+
 	  hex->data[byte_offset] = event->string[0];
 
 	  hex_update_current(hex, byte_offset);
@@ -726,7 +762,7 @@ hex_text_data_key_press(GtkHex *hex, GdkEventKey *event, int byte_offset)
 		      byte_offset, 1);
     }
 
-  return TRUE;
+  return FALSE;
 }
 
 
@@ -765,7 +801,9 @@ on_hex_text_key_press_event        (GtkWidget       *widget,
   byte_offset = hex_cursor_index_to_byte_index(hex);
   gtk_signal_emit(GTK_OBJECT(hex), hex_signals[MOVE_CURSOR], byte_offset);
 
-  return FALSE;
+  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+
+  return TRUE;
 }
 
 
@@ -795,6 +833,38 @@ on_hex_button_press_event     (GtkWidget       *widget,
 }
 
 
+static int
+hex_get_largest_hex_index(GtkHex *hex)
+{
+  int full_lines;
+  int offset;
+
+  if (!hex)
+    return 0;
+
+  full_lines = hex->data_size / HEX_LINE_BYTES;
+  offset = hex->data_size % HEX_LINE_BYTES;
+
+  return full_lines * HEX_LINE_WIDTH + HEX_LINE_START + 1 + (offset - 1 ) * 3 + (offset > 8 ? 1 : 0);
+}
+
+
+static int
+hex_get_largest_ascii_index(GtkHex *hex)
+{
+  int full_lines;
+  int offset;
+
+  if (!hex)
+    return 0;
+
+  full_lines = hex->data_size / HEX_LINE_BYTES;
+  offset = hex->data_size % HEX_LINE_BYTES;
+  
+  return full_lines * HEX_LINE_WIDTH + HEX_LINE_START_ASCII + offset - 1 + (offset > 8 ? 1 : 0);
+}
+
+
 /* Assuming the cursor would be placed at INDEX, is that
  * a feasible (i.e. hex or ascii, but not whitespace) index?
  * If so, the same index is returned,
@@ -802,11 +872,11 @@ on_hex_button_press_event     (GtkWidget       *widget,
  * non-whitespace.
  */
 static int
-hex_get_nowhite_index(GtkHex *hex, int index_orig)
+hex_get_nowhite_index(GtkHex *hex, int index)
 {
   GtkEditable *ed;
-  int line_offset;
-  int index;
+  int line_offset, last_line, line;
+  int offset_hex_max, offset_ascii_max;
 
   if (!hex)
     return FALSE;
@@ -814,10 +884,15 @@ hex_get_nowhite_index(GtkHex *hex, int index_orig)
   ed = GTK_EDITABLE(hex->hex_text);
 
   if (hex->mode == GTK_RAW_ASCII)
-    return index_orig;
+    return index;
 
-  line_offset = index_orig % HEX_LINE_WIDTH;
-  index = (index_orig / HEX_LINE_WIDTH) * HEX_LINE_WIDTH + line_offset;
+  line_offset = index % HEX_LINE_WIDTH;
+
+  offset_hex_max   = hex_get_largest_hex_index(hex) % HEX_LINE_WIDTH;
+  offset_ascii_max = hex_get_largest_ascii_index(hex) % HEX_LINE_WIDTH;  
+
+  line = index / HEX_LINE_WIDTH;
+  last_line = hex_get_largest_hex_index(hex) / HEX_LINE_WIDTH;
 
   if (line_offset < HEX_LINE_START)
     {
@@ -825,8 +900,19 @@ hex_get_nowhite_index(GtkHex *hex, int index_orig)
       return index;
     }
 
+  /* Below, we need to check if there are actually enough bytes in the
+   * requested line -- it could be that we're in the last row
+   * where we may have less!
+   */
+
   if (line_offset >= HEX_LINE_START && line_offset <= HEX_LINE_END)
     {
+      if (line == last_line)
+	{
+	  line_offset = MIN(line_offset, offset_hex_max);
+	  index = (index / HEX_LINE_WIDTH) * HEX_LINE_WIDTH + line_offset;
+	}
+
       if (line_offset <= HEX_LINE_LEFT_MIDDLE)
 	{
 	  if ((line_offset - HEX_LINE_START) % 3 == 2)
@@ -848,11 +934,15 @@ hex_get_nowhite_index(GtkHex *hex, int index_orig)
   
   if (line_offset > HEX_LINE_END && line_offset < HEX_LINE_START_ASCII)
     {
-      index -= (line_offset % HEX_LINE_END);
+      if (line == last_line)
+	line_offset = MIN(line_offset, offset_hex_max);
+
+      index = (index / HEX_LINE_WIDTH) * HEX_LINE_WIDTH + line_offset;
+
       return index;
     }
 
-  return index;
+  return MIN(index, hex_get_largest_ascii_index(hex));
 }
 
 
@@ -1083,7 +1173,7 @@ hex_get_hex_text(const guchar *data, int data_size)
  * The string is returned through the result pointer.
  * An additional string will be returned through null_map,
  * that contains a null character "0" wherever the original text
-'* contained a null byte, and whitespace everywhere else.
+ * contained a null byte, and whitespace everywhere else.
  * The strings must be freed when it's no longer needed.
  */
 void
@@ -1099,6 +1189,7 @@ hex_get_ascii_text(const guchar *data, int data_size,
 
   *result = result_ptr = g_new0(guchar, data_size + 1);
   *null_map = null_map_ptr = g_new0(guchar, data_size + 1);
+  memset(null_map_ptr, ' ', data_size);
 
   if (!result || !null_map)
     {
@@ -1109,7 +1200,7 @@ hex_get_ascii_text(const guchar *data, int data_size,
 
   for (data_ptr = data; data_ptr < data + data_size; result_ptr++, data_ptr++, null_map_ptr++)
     {
-      int val = *data_ptr;
+      char val = *data_ptr;
 
       if (hex_is_char_unprintable(val))
 	{
@@ -1117,7 +1208,7 @@ hex_get_ascii_text(const guchar *data, int data_size,
 	    {
 	    case '\n':
 	      *result_ptr = *data_ptr;
-	      *null_map_ptr = ' ';
+	      *null_map_ptr = '\n';
 	      break;
 
 	    case 0:
@@ -1127,13 +1218,12 @@ hex_get_ascii_text(const guchar *data, int data_size,
 
 	    default:
 	      *result_ptr = NONPRINT_CHAR;
-	      *null_map_ptr = val;
+	      *null_map_ptr = *data_ptr;
 	    }
 	}
       else
 	{
 	  *result_ptr = *data_ptr;
-	  *null_map_ptr = ' ';
 	}
     }
 }
@@ -1155,24 +1245,18 @@ hex_sync_data_to_ascii(GtkHex *hex)
   memcpy(hex->data, text, MIN(len, hex->data_size));
   g_free(text);
 
-  text = gtk_editable_get_chars(GTK_EDITABLE(hex->priv->null_map), 0, -1);
+  text = hex->priv->null_map;
 
   for (i = 0; i < MIN(len, hex->data_size); i++)
     {
       if (text[i] != ' ')
 	{
-	  if (text[i] == '0')
-	    {
-	      hex->data[i] = 0;
-	    }
+	  if (text[i] == '0')	    
+	    hex->data[i] = 0;
 	  else
-	    {
-	      hex->data[i] = text[i];
-	    }
+	    hex->data[i] = text[i];
 	}
     }
-
-  g_free(text);
 }
 
 
@@ -1186,7 +1270,7 @@ gtk_hex_destroy(GtkObject *object)
 
   hex = GTK_HEX(object);
 
-  gtk_widget_unref(GTK_WIDGET(hex->priv->null_map));
+  g_free(hex->priv->null_map);
   g_free(hex->priv);
   
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
@@ -1273,7 +1357,6 @@ gtk_hex_init (GtkHex *hex)
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (hex->mode_button_ascii);
   gtk_box_pack_start (GTK_BOX (mode_hbox), hex->mode_button_ascii, FALSE, FALSE, 0);
-
   scrolledwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_ref (scrolledwin);
   gtk_object_set_data_full (GTK_OBJECT (hex_vbox), "scrolledwin", scrolledwin,
@@ -1294,12 +1377,11 @@ gtk_hex_init (GtkHex *hex)
   gtk_text_set_line_wrap(GTK_TEXT(hex->hex_text), FALSE);
 
   hex->mode = GTK_RAW_HEX;
-
+  
   priv = g_new0(GtkHexPrivate, 1);
-  priv->null_map = GTK_TEXT(gtk_text_new(NULL, NULL));
-  gtk_widget_ref(GTK_WIDGET(priv->null_map));
-  gtk_object_set_data_full (GTK_OBJECT (hex_vbox), "null_map", priv->null_map,
-                            (GtkDestroyNotify) gtk_widget_unref);
+  priv->null_map = g_new0(gchar, 1024);
+  priv->null_map_used = 0;
+  priv->null_map_len  = 1024;
   hex->priv = priv;
 
   gtk_signal_connect (GTK_OBJECT (hex->mode_button_ascii), "toggled",
@@ -1391,9 +1473,8 @@ gtk_hex_set_content(GtkHex *hex, char *data, int data_size)
       if (!hex_text || !null_map_text)
 	goto cleanup;
 
-      gtk_editable_delete_text(GTK_EDITABLE(hex->priv->null_map), 0, -1);
-      gtk_text_insert(GTK_TEXT(hex->priv->null_map), NULL, NULL, NULL,
-		      null_map_text, strlen(null_map_text));
+      null_map_delete(hex->priv, 0, -1);
+      null_map_insert(hex->priv, null_map_text, 0, strlen(null_map_text));
       break;
       
     default:
