@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -28,20 +28,14 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-#ifdef LINUX
-#define __FAVOR_BSD
-#endif
-#include <netinet/in_systm.h>
-#include <netinet/in.h>
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 4096
-#endif
+#include <libnd.h>
+#include <gtk/gtk.h>
 
-#include <netdude/nd_globals.h>
-#include <netdude/nd_types.h>
-#include <netdude/nd_macros.h>
-#include <netdude/nd_main.h>
-#include <netdude/nd_debug.h>
+#include <nd_debug.h>
+#include <nd_globals.h>
+#include <nd_macros.h>
+#include <nd_types.h>
+#include <nd_gettext.h>
 
 #endif
