@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -33,16 +33,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <errno.h>
 
 #include <gtk/gtk.h>
-#include <netdude/nd.h>
-#include <netdude/nd_gui.h>
-#include <netdude/nd_globals.h>
-#include <netdude/nd_packet.h>
-#include <netdude/nd_misc.h>
-#include <netdude/nd_main.h>
-#include <netdude/nd_prefs.h>
-#include <netdude/nd_trace_registry.h>
-#include <netdude/nd_dialog.h>
-#include <netdude/nd_timestamp.h>
+#include <nd.h>
+#include <nd_gui.h>
+#include <nd_main.h>
+#include <nd_trace_registry.h>
+#include <nd_dialog.h>
+#include <nd_timestamp.h>
 #include <callbacks.h>
 #include <interface.h>
 #include <support.h>
@@ -52,7 +48,7 @@ typedef struct nd_hardware_dialog_data
   int                        address_len;
   ND_AddressCallback         callback_okay;
   ND_DialogCallback          callback_cancel;
-  ND_Packet                 *packet;
+  LND_Packet                *packet;
   void                      *user_data;
 } ND_HardwareDialogData;
 
@@ -67,7 +63,7 @@ dialog_largenumber(const char *message,
 		   guint value, 
 		   ND_NumberCallback callback_okay,
 		   ND_DialogCallback callback_cancel,
-		   ND_Packet *packet,
+		   LND_Packet *packet,
 		   void *user_data);
 
 
@@ -100,7 +96,7 @@ hw_okay_clicked                    (GtkButton       *button,
   for (i = 0; i < dialog_data->address_len; i++)
     {
       g_snprintf(key, MAXPATHLEN, "hw_byte_%i", i);
-      entry = gtk_object_get_data(GTK_OBJECT(dialog), key);
+      ND_GTK_GET(entry, dialog, key);
       address[i] = (guchar) strtoul(gtk_entry_get_text(GTK_ENTRY(entry)), NULL, 16);				    
     }
 
@@ -470,10 +466,10 @@ nd_dialog_number(const char *message,
 		 guint value, guint max,
 		 ND_NumberCallback callback_okay,
 		 ND_DialogCallback callback_cancel,
-		 ND_Packet *packet,
+		 LND_Packet *packet,
 		 void *user_data)
 {
-  ND_Trace       *trace;
+  LND_Trace      *trace;
   GtkWidget      *w;
   GtkObject      *adj;
 
@@ -498,12 +494,11 @@ nd_dialog_number(const char *message,
     }
   else
     {
-      adj = gtk_object_get_data(GTK_OBJECT(number_dialog), "adj");
+      ND_GTK_GET(adj, number_dialog, "adj");
       GTK_ADJUSTMENT(adj)->upper = max;
     }
 
-  w = gtk_object_get_data(GTK_OBJECT(number_dialog), "data_entry_label");
-  D_ASSERT_PTR(w);
+  ND_GTK_GET(w, number_dialog, "data_entry_label");
   gtk_label_set_text(GTK_LABEL(w), message);
   
   gtk_object_set_data(GTK_OBJECT(number_dialog), "callback_okay", callback_okay);
@@ -544,12 +539,10 @@ nd_dialog_number_update(void)
   else
     g_snprintf(s, MAXPATHLEN, "0x%x", value);	       
 
-  w = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_entry");
-  D_ASSERT_PTR(w);
+  ND_GTK_GET(w, number_dialog, "generic_entry");
   gtk_entry_set_text(GTK_ENTRY(w), s);
 
-  w = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_spin");
-  D_ASSERT_PTR(w);  
+  ND_GTK_GET(w, number_dialog, "generic_spin");
   gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(w), GTK_ADJUSTMENT(adj));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), value);
 
@@ -570,14 +563,10 @@ nd_dialog_number_set_base(ND_Base base)
 
   gtk_object_set_data(GTK_OBJECT(number_dialog), "base", GINT_TO_POINTER(base));
 
-  spin = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_spin");
-  D_ASSERT_PTR(spin);
-  entry = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_entry");
-  D_ASSERT_PTR(entry);
-  dec_toggle = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_decimal");
-  D_ASSERT_PTR(dec_toggle);
-  hex_toggle = gtk_object_get_data(GTK_OBJECT(number_dialog), "generic_hex");
-  D_ASSERT_PTR(hex_toggle);
+  ND_GTK_GET(spin, number_dialog, "generic_spin");
+  ND_GTK_GET(entry, number_dialog, "generic_entry");
+  ND_GTK_GET(dec_toggle, number_dialog, "generic_decimal");
+  ND_GTK_GET(hex_toggle, number_dialog, "generic_hex");
 
   switch (base)
     {
@@ -616,17 +605,18 @@ nd_dialog_number_set_base(ND_Base base)
 void      
 nd_dialog_number_ok(GtkWidget *dialog)
 {
-  ND_Base    base;
+  ND_Base     base;
   ND_NumberCallback callback;
-  ND_Packet *packet;
-  GtkWidget *w;
-  GtkWidget *entry;
-  void      *user;
-  guint      value;
-  char      *errptr = NULL;
+  LND_Packet *packet;
+  GtkWidget  *w;
+  GtkWidget  *entry;
+  void       *user;
+  guint       value;
+  char       *errptr = NULL;
 
-  w = gtk_object_get_data(GTK_OBJECT(dialog), "generic_spin");
-  D_ASSERT_PTR(w);
+  D_ENTER;
+
+  ND_GTK_GET(w, dialog, "generic_spin");
 
   callback  = gtk_object_get_data(GTK_OBJECT(dialog), "callback_okay");
   user      = gtk_object_get_data(GTK_OBJECT(dialog), "user_data");
@@ -636,27 +626,26 @@ nd_dialog_number_ok(GtkWidget *dialog)
   switch(base)
     {
     case ND_BASE_HEX:
-      entry = gtk_object_get_data(GTK_OBJECT(dialog), "generic_entry");
-      D_ASSERT_PTR(entry);
-
+      ND_GTK_GET(entry, dialog, "generic_entry");
       value = strtol(gtk_entry_get_text(GTK_ENTRY(entry)), &errptr, 16);
       break;
 
     case ND_BASE_DEC:
     default:
-      entry = gtk_object_get_data(GTK_OBJECT(dialog), "generic_spin");
-      D_ASSERT_PTR(entry);
-
+      ND_GTK_GET(entry, dialog, "generic_spin");
       value = strtol(gtk_entry_get_text(GTK_ENTRY(entry)), &errptr, 10);
     }
 
   if (*errptr == '\0')
     {
+      D(("Calling callback with value %u\n", value));
       gtk_widget_hide(dialog);  
       
       if (callback)
 	callback(packet, user, value);
     }
+
+  D_RETURN;
 }
 
 
@@ -681,7 +670,7 @@ dialog_largenumber(const char *message,
 		   guint value, 
 		   ND_NumberCallback callback_ok,
 		   ND_DialogCallback callback_cancel,
-		   ND_Packet *packet,
+		   LND_Packet *packet,
 		   void *user_data)
 {
   static GtkWidget *dialog = NULL;
@@ -693,8 +682,7 @@ dialog_largenumber(const char *message,
     {
       dialog = create_largenumber_dialog ();
       
-      w = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_up");
-      D_ASSERT_PTR(w);
+      ND_GTK_GET(w, dialog, "largenumber_up");
       arrow = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_NONE);
       gtk_widget_show(arrow);
       gtk_widget_ref(arrow);
@@ -703,8 +691,7 @@ dialog_largenumber(const char *message,
       gtk_container_remove(GTK_CONTAINER(w), GTK_BIN(w)->child);
       gtk_container_add(GTK_CONTAINER(w), arrow);
       
-      w = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_down");
-      D_ASSERT_PTR(w);
+      ND_GTK_GET(w, dialog, "largenumber_down");
       arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_NONE);
       gtk_widget_show(arrow);
       gtk_widget_ref(arrow);
@@ -716,13 +703,12 @@ dialog_largenumber(const char *message,
 
   if (message)
     {
-      w = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_label");
-      D_ASSERT_PTR(w);
+      ND_GTK_GET(w, dialog, "largenumber_label");
       gtk_label_set_text(GTK_LABEL(w), message);
     }
 
   g_snprintf(s, MAXPATHLEN, "%u", value);
-  entry = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_entry");
+  ND_GTK_GET(entry, dialog, "largenumber_entry");
   gtk_entry_set_text(GTK_ENTRY(entry), s);
 
   gtk_object_set_data(GTK_OBJECT(dialog), "callback_okay", callback_ok);
@@ -738,11 +724,11 @@ void
 nd_dialog_largenumber_ok(GtkWidget *dialog)
 {
   ND_NumberCallback callback_okay;
-  ND_Packet *packet;
-  GtkWidget *entry;
-  char      *errptr;
-  guint      value;
-  void      *user_data;
+  LND_Packet *packet;
+  GtkWidget  *entry;
+  char       *errptr;
+  guint       value;
+  void       *user_data;
 
   if (!dialog)
     return;
@@ -751,9 +737,7 @@ nd_dialog_largenumber_ok(GtkWidget *dialog)
   user_data = gtk_object_get_data(GTK_OBJECT(dialog), "user_data");
   packet = gtk_object_get_data(GTK_OBJECT(dialog), "packet");
 
-  entry = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_entry");
-  D_ASSERT_PTR(entry);
-
+  ND_GTK_GET(entry, dialog, "largenumber_entry");
   value = strtoul(gtk_entry_get_text(GTK_ENTRY(entry)), &errptr, 10);
       
   if ((*errptr != '\0') || (errno == ERANGE))
@@ -777,9 +761,7 @@ nd_dialog_largenumber_up(GtkWidget *dialog)
   if (!dialog)
     return;
 
-  entry = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_entry");
-  D_ASSERT_PTR(entry);
-
+  ND_GTK_GET(entry, dialog, "largenumber_entry");
   value = strtoul(gtk_entry_get_text(GTK_ENTRY(entry)), &errptr, 10);
       
   if ((*errptr != '\0') || (errno == ERANGE))
@@ -801,9 +783,7 @@ nd_dialog_largenumber_down(GtkWidget *dialog)
   if (!dialog)
     return;
 
-  entry = gtk_object_get_data(GTK_OBJECT(dialog), "largenumber_entry");
-  D_ASSERT_PTR(entry);
-
+  ND_GTK_GET(entry, dialog, "largenumber_entry");
   value = strtoul(gtk_entry_get_text(GTK_ENTRY(entry)), &errptr, 10);
       
   if ((*errptr != '\0') || (errno == ERANGE))
@@ -814,12 +794,90 @@ nd_dialog_largenumber_down(GtkWidget *dialog)
 }
 
 
+void      
+nd_dialog_string(const char *title,
+		 const char *message,
+		 ND_StringCallback callback_okay,
+		 ND_DialogCallback callback_cancel,
+		 void *user_data)
+{
+  static GtkWidget *string_dialog;
+  GtkWidget *w;
+
+  if (!string_dialog)
+    string_dialog = create_string_dialog();
+
+  /* Reset entry field */
+  ND_GTK_GET(w, string_dialog, "string_entry");
+  gtk_entry_set_text(GTK_ENTRY(w), "");
+  /* Make sure the entry area has the focus: */
+  gtk_widget_grab_focus(w);
+
+  /* Set dialog title, if given */
+  if (title)
+    gtk_window_set_title(GTK_WINDOW(string_dialog), title);
+
+  /* Set dialog message, if given */
+  if (message)
+    {
+      ND_GTK_GET(w, string_dialog, "data_entry_label");
+      gtk_label_set_text(GTK_LABEL(w), message);
+    }
+  
+  gtk_object_set_data(GTK_OBJECT(string_dialog), "callback_okay", callback_okay);
+  gtk_object_set_data(GTK_OBJECT(string_dialog), "callback_cancel", callback_cancel);
+  gtk_object_set_data(GTK_OBJECT(string_dialog), "user_data", user_data);
+  
+  gtk_widget_show(string_dialog);
+}
+
+
+void      
+nd_dialog_string_ok(GtkWidget *dialog)
+{
+  ND_StringCallback callback;
+  void *user;
+  char *string;
+  GtkWidget *w;
+
+  ND_GTK_GET(w, dialog, "string_entry");
+
+  callback  = gtk_object_get_data(GTK_OBJECT(dialog), "callback_okay");
+  user      = gtk_object_get_data(GTK_OBJECT(dialog), "user_data");
+  string    = gtk_entry_get_text(GTK_ENTRY(w));
+
+  if (!string || strlen(string) == 0)
+    return;
+
+  if (callback)
+    callback(string, user);
+  
+  gtk_widget_hide(dialog);
+}
+
+
+void      
+nd_dialog_string_cancel(GtkWidget *dialog)
+{
+  ND_DialogCallback callback;
+  void *user;
+
+  callback  = gtk_object_get_data(GTK_OBJECT(dialog), "callback_cancel");
+  user      = gtk_object_get_data(GTK_OBJECT(dialog), "user_data");
+  
+  if (callback)
+    callback(user);
+  
+  gtk_widget_hide(dialog);
+}
+
+
 void      
 nd_dialog_hardware_address(const char *message,
 			   guchar *address, int address_len,
 			   ND_AddressCallback callback_okay,
 			   ND_DialogCallback callback_cancel,
-			   ND_Packet *packet,
+			   LND_Packet *packet,
 			   void *user_data)
 {
   ND_HardwareDialogData  *data;
@@ -844,8 +902,7 @@ nd_dialog_hardware_address(const char *message,
   for (i = 0; i < address_len; i++)
     {
       g_snprintf(key, MAXPATHLEN, "hw_byte_%i", i);
-      w = gtk_object_get_data(GTK_OBJECT(dialog), key);
-      D_ASSERT_PTR(w);
+      ND_GTK_GET(w, dialog, key);
       sprintf(val, "%.2x", address[i]);
       gtk_entry_set_text(GTK_ENTRY(w), val);      
     }
@@ -858,7 +915,7 @@ void
 nd_dialog_ip(const char *message, guchar *ip_address,
 	     ND_AddressCallback callback_ok,
 	     ND_DialogCallback  callback_cancel,
-	     ND_Packet         *packet,
+	     LND_Packet         *packet,
 	     void              *user_data)
 {
   GtkLabel *label;
@@ -876,28 +933,22 @@ nd_dialog_ip(const char *message, guchar *ip_address,
       gtk_object_set_data(GTK_OBJECT(ip_dialog), "ip_address", ip_address_buf);
     }
 
-  ip_address_buf = gtk_object_get_data(GTK_OBJECT(ip_dialog), "ip_address");
+  ND_GTK_GET(ip_address_buf, ip_dialog, "ip_address");
   memcpy(ip_address_buf, ip_address, 4*sizeof(char));
 
-  label = gtk_object_get_data(GTK_OBJECT(ip_dialog), "ip_label");
-  D_ASSERT_PTR(label);
-
+  ND_GTK_GET(label, ip_dialog, "ip_label");
   gtk_label_set_text(label, message);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton1");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton1");
   gtk_spin_button_set_value(sb, ip_address[0]);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton2");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton2");
   gtk_spin_button_set_value(sb, ip_address[1]);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton3");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton3");
   gtk_spin_button_set_value(sb, ip_address[2]);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton4");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton4");
   gtk_spin_button_set_value(sb, ip_address[3]);
 
   gtk_object_set_data(GTK_OBJECT(ip_dialog), "data", user_data);
@@ -914,7 +965,7 @@ nd_dialog_ip_ok(void)
 {
   ND_AddressCallback  callback_ok;
   void               *data;
-  ND_Packet          *packet;
+  LND_Packet         *packet;
   guchar             *ip_address_buf;
   GtkSpinButton      *sb;
 
@@ -926,20 +977,16 @@ nd_dialog_ip_ok(void)
   callback_ok = gtk_object_get_data(GTK_OBJECT(ip_dialog), "callback_ok");
   ip_address_buf = gtk_object_get_data(GTK_OBJECT(ip_dialog), "ip_address");
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton1");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton1");
   ip_address_buf[0] = (guchar) gtk_spin_button_get_value_as_int(sb);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton2");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton2");
   ip_address_buf[1] = (guchar) gtk_spin_button_get_value_as_int(sb);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton3");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton3");
   ip_address_buf[2] = (guchar) gtk_spin_button_get_value_as_int(sb);
 
-  sb = gtk_object_get_data(GTK_OBJECT(ip_dialog), "spinbutton4");
-  D_ASSERT_PTR(sb);
+  ND_GTK_GET(sb, ip_dialog, "spinbutton4");
   ip_address_buf[3] = (guchar) gtk_spin_button_get_value_as_int(sb);
 
   gtk_widget_hide(ip_dialog);
@@ -973,7 +1020,7 @@ nd_dialog_ip_cancel(void)
 void
 nd_dialog_filesel(const char *title,
 		  const char *fname,
-		  ND_FilenameCallback callback_fname,
+		  ND_StringCallback callback_fname,
 		  void *user_data)
 {
   if (!callback_fname)
@@ -996,7 +1043,7 @@ nd_dialog_filesel(const char *title,
 void
 nd_dialog_filesel_ok(GtkWidget *dialog)
 {
-  ND_FilenameCallback  callback;
+  ND_StringCallback   callback;
   char               *filename;
   void               *user_data;
 
@@ -1030,8 +1077,9 @@ nd_dialog_filesel_close(void)
 static void 
 dialog_exit_trace_saved(void *user_data)
 {
-  GtkWidget *entry_hbox = gtk_object_get_data(GTK_OBJECT(user_data), "entry_hbox");
+  GtkWidget *entry_hbox;
 
+  ND_GTK_GET(entry_hbox, user_data, "entry_hbox");
   gtk_widget_set_sensitive(entry_hbox, FALSE);
 }
 
@@ -1043,7 +1091,7 @@ static void
 dialog_exit_save_clicked(GtkButton *button,
                          gpointer  user_data)
 {
-  ND_Trace *trace = (ND_Trace *) user_data;
+  LND_Trace *trace = (LND_Trace *) user_data;
 
   nd_trace_save_as_dialog(trace, dialog_exit_trace_saved, button);
 
@@ -1053,7 +1101,7 @@ dialog_exit_save_clicked(GtkButton *button,
 
 
 static GtkWidget *
-dialog_exit_create_entry(ND_Trace *trace)
+dialog_exit_create_entry(LND_Trace *trace)
 {
   GtkWidget *entry_hbox;
   GtkWidget *filename_label;
@@ -1063,7 +1111,7 @@ dialog_exit_create_entry(ND_Trace *trace)
   gtk_widget_ref (entry_hbox);
   gtk_widget_show (entry_hbox);
 
-  filename_label = gtk_label_new (nd_trace_get_name(trace));
+  filename_label = gtk_label_new (libnd_trace_get_name(trace));
   gtk_widget_ref (filename_label);
   gtk_object_set_data_full (GTK_OBJECT (entry_hbox), "filename_label", filename_label,
                             (GtkDestroyNotify) gtk_widget_unref);
@@ -1094,7 +1142,7 @@ dialog_exit_create_entry(ND_Trace *trace)
 /* This is called once for each loaded trace, to set up
    the vbox containing all the modified traces. */
 static void 
-dialog_exit_trace_cb(ND_Trace *trace, void *user_data)
+dialog_exit_trace_cb(LND_Trace *trace, void *user_data)
 {
   GtkWidget *vbox = (GtkWidget *) user_data;
   GtkWidget *entry_hbox;
@@ -1115,7 +1163,7 @@ dialog_exit_trace_cb(ND_Trace *trace, void *user_data)
 static void
 dialog_unsafe_exit_cb(void *user_data)
 {
-  nd_prefs_save();
+  libnd_prefs_save();
   exit(0);
   TOUCH(user_data);
 }
@@ -1150,16 +1198,19 @@ nd_dialog_about(void)
   static GtkWidget *dialog = NULL;
   static char *about_text =N_("\n"
 			      "   Programming:\n"
-			      "        Christian Kreibich <christian@whoop.org>\n"
-			      "\n"
-			      "   Packagers:\n"
-			      "        Noel Koethe (Debian)\n"
+			      "     Christian Kreibich <christian.kreibich-AT-cl.cam.ac.uk>\n"
 			      "\n"
 			      "   Help, Feedback,\n"
 			      "   Suggestions, Inspiration:\n"
-			      "        Mark Handley\n"
-			      "        Vern Paxson\n"
-			      "        Daniel Stodden");
+			      "     Mark Handley\n"
+			      "     Vern Paxson\n"
+			      "     Daniel Stodden\n"
+			      "     Euan Harris\n"
+			      "     Andrew Moore\n"
+			      "\n"
+			      "   Yet Another SRG Production\n"
+			      "   http://www.cl.cam.ac.uk/Research/SRG\n\n"
+			      "   ");
 
   GtkWidget *label;
 
@@ -1170,12 +1221,9 @@ nd_dialog_about(void)
 
       dialog = create_about_dialog();
       /* HORRIBLE workaround for glade segfault! */
-      w = gtk_object_get_data(GTK_OBJECT(dialog), "about_text");
-      D_ASSERT_PTR(w);
-      gtk_editable_insert_text(GTK_EDITABLE(w),
-			       about_text,
-			       strlen(about_text),
-			       &pos);
+      ND_GTK_GET(w, dialog, "about_text");      
+      gtk_editable_insert_text(GTK_EDITABLE(w), about_text,
+			       strlen(about_text), &pos);
     }
   
   if ((label = gtk_object_get_data(GTK_OBJECT(dialog), "about_version_label")))
@@ -1200,7 +1248,7 @@ nd_dialog_about(void)
 
 void      
 nd_dialog_fontsel(const char *title, const char *font_name,
-		  ND_FontselCallback callback_ok,
+		  ND_StringCallback callback_ok,
 		  void *user_data)
 {
  static GtkWidget *dialog = NULL;
@@ -1241,7 +1289,7 @@ nd_dialog_fontsel_cancel(GtkFontSelectionDialog *dialog)
 void      
 nd_dialog_fontsel_apply(GtkFontSelectionDialog *dialog)
 {
-  ND_FontselCallback  callback;
+  ND_StringCallback   callback;
   void               *user_data;
   char               *font_name;
 
