@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -33,13 +33,11 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <string.h>
 #include <math.h>
 
-#include <netdude/nd.h>
-#include <netdude/nd_globals.h>
-#include <netdude/nd_gui.h>
-#include <netdude/nd_misc.h>
-#include <netdude/nd_prefs.h>
-#include <netdude/nd_trace_registry.h>
-#include <netdude/nd_recent.h>
+#include <nd.h>
+#include <nd_gui.h>
+#include <nd_prefs.h>
+#include <nd_trace_registry.h>
+#include <nd_recent.h>
 
 
 static void
@@ -53,7 +51,7 @@ recent_change_menuitem(int item, const char *label)
     return;
 
   sprintf(s, "recent_item%i", item);
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), s);
+  ND_GTK_GET(w, nd_toplevel_window, s);
 
   if (!label)
     {
@@ -67,13 +65,13 @@ recent_change_menuitem(int item, const char *label)
   else
     g_snprintf(s, MAXPATHLEN, "%i. %s", item+1, label);
 
-  if ((w2 = gtk_object_get_data(GTK_OBJECT(toplevel), "tooltips")))
-    gtk_tooltips_set_tip(GTK_TOOLTIPS(w2), w, label, NULL);
+  ND_GTK_GET(w2, nd_toplevel_window, "tooltips");
+  gtk_tooltips_set_tip(GTK_TOOLTIPS(w2), w, label, NULL);
   
   gtk_widget_show(w);
   gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child), s);
 
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_separator");
+  ND_GTK_GET(w, nd_toplevel_window, "recent_separator");
   gtk_widget_show(w);
 }
 
@@ -83,20 +81,24 @@ nd_recent_init(void)
 {
   GtkWidget * w;
 
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_separator")))
-    gtk_widget_hide(w);
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_item0")))
-    gtk_widget_hide(w);
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_item1")))
-    gtk_widget_hide(w);
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_item2")))
-    gtk_widget_hide(w);
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_item3")))
-    gtk_widget_hide(w);
-  if ((w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_item4")))
-    gtk_widget_hide(w);
+  D_ENTER;
+
+  ND_GTK_GET(w, nd_toplevel_window, "recent_separator");
+  gtk_widget_hide(w);
+
+  ND_GTK_GET(w, nd_toplevel_window, "recent_item0");
+  gtk_widget_hide(w);
+  ND_GTK_GET(w, nd_toplevel_window, "recent_item1");
+  gtk_widget_hide(w);
+  ND_GTK_GET(w, nd_toplevel_window, "recent_item2");
+  gtk_widget_hide(w);
+  ND_GTK_GET(w, nd_toplevel_window, "recent_item3");
+  gtk_widget_hide(w);
+  ND_GTK_GET(w, nd_toplevel_window, "recent_item4");
+  gtk_widget_hide(w);
 
   nd_recent_reload_entries();
+  D_RETURN;
 }
 
 
@@ -108,7 +110,7 @@ nd_recent_reload_entries(void)
   char      *recent_file;
   int        i, last;
 
-  w = gtk_object_get_data(GTK_OBJECT(toplevel), "recent_separator");
+  ND_GTK_GET(w, nd_toplevel_window, "recent_separator");
   gtk_widget_hide(w);
 
   for (i = 0, last = 0; i < NUM_RECENT; i++)
@@ -116,13 +118,13 @@ nd_recent_reload_entries(void)
       recent_change_menuitem(i, NULL);
       sprintf(key, "recent_%i", i);
 
-      if (nd_prefs_get_str_item(ND_DOM_NETDUDE, key, &recent_file))
+      if (libnd_prefs_get_str_item(ND_DOM_NETDUDE, key, &recent_file))
 	{
-	  if (nd_misc_is_tcpdump_file(recent_file))
+	  if (libnd_misc_is_tcpdump_file(recent_file))
 	    {
 	      recent_change_menuitem(last, recent_file);
 	      sprintf(key, "recent_%i", last);
-	      nd_prefs_set_str_item(ND_DOM_NETDUDE, key, recent_file);
+	      libnd_prefs_set_str_item(ND_DOM_NETDUDE, key, recent_file);
 	      last++;
 	    }
 	}
@@ -132,7 +134,7 @@ nd_recent_reload_entries(void)
   for ( ; last < NUM_RECENT; last++)
     {
       sprintf(key, "recent_%i", last);
-      nd_prefs_del_item(ND_DOM_NETDUDE, key);
+      libnd_prefs_del_item(ND_DOM_NETDUDE, key);
     }
 }
 
@@ -153,7 +155,7 @@ nd_recent_add_file(const char *filename)
     {
       g_snprintf(key, MAXPATHLEN, "recent_%i", i);
       
-      if (nd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
+      if (libnd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
 	{
 	  if (!strcmp(filename, file))
 	    {
@@ -166,17 +168,17 @@ nd_recent_add_file(const char *filename)
 		{
 		  g_snprintf(key, MAXPATHLEN, "recent_%i", j);
 
-		  if (nd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
+		  if (libnd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
 		    {
 		      recent_change_menuitem(j + 1, file);
 		      g_snprintf(key, MAXPATHLEN, "recent_%i", j + 1);
-		      nd_prefs_set_str_item(ND_DOM_NETDUDE, key, file);
+		      libnd_prefs_set_str_item(ND_DOM_NETDUDE, key, file);
 		    }
 		}
 
 	      recent_change_menuitem(0, filename);
-	      nd_prefs_set_str_item(ND_DOM_NETDUDE, "recent_0", filename);      
-	      nd_prefs_save_general();
+	      libnd_prefs_set_str_item(ND_DOM_NETDUDE, "recent_0", filename);      
+	      libnd_prefs_save();
 	      return;
 	    }
 	}
@@ -188,17 +190,17 @@ nd_recent_add_file(const char *filename)
     {
       g_snprintf(key, MAXPATHLEN, "recent_%i", i);
       
-      if (nd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
+      if (libnd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
 	{
 	  recent_change_menuitem(i+1, file);
 	  g_snprintf(key, MAXPATHLEN, "recent_%i", i+1);
-	  nd_prefs_set_str_item(ND_DOM_NETDUDE,key, file);
+	  libnd_prefs_set_str_item(ND_DOM_NETDUDE,key, file);
 	}
     }
   
-  nd_prefs_set_str_item(ND_DOM_NETDUDE, "recent_0", (void*) filename);
+  libnd_prefs_set_str_item(ND_DOM_NETDUDE, "recent_0", (void*) filename);
   recent_change_menuitem(0, filename);
-  nd_prefs_save_general();
+  libnd_prefs_save();
 }
 
 
@@ -213,9 +215,9 @@ nd_recent_load(int index)
 
   g_snprintf(key, MAXPATHLEN, "recent_%i", index);
 
-  if (nd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
+  if (libnd_prefs_get_str_item(ND_DOM_NETDUDE, key, &file))
     {
-      ND_Trace *trace;
+      LND_Trace *trace;
 
       trace = nd_trace_new(file);
       D_ASSERT_PTR(trace);
