@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2001, 2002 Christian Kreibich <christian@whoop.org>.
+Copyright (C) 2000 - 2004 Christian Kreibich <christian@whoop.org>.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -26,13 +26,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define __nd_types_h
 
 typedef struct nd_trace ND_Trace;
-typedef struct nd_packet ND_Packet;
 typedef struct nd_protocol ND_Protocol;
 typedef struct nd_protocol_field ND_ProtoField;
 typedef struct nd_menu_data ND_MenuData;
-typedef struct nd_proto_inst ND_ProtoInst;
-typedef struct nd_proto_data ND_ProtoData;
 typedef struct nd_proto_info ND_ProtoInfo;
-
+typedef struct nd_filter ND_Filter;
 
 #endif
